@@ -155,6 +155,11 @@
     nil))
 
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defmacro list-cons-when (test car cdr)
+    `(when ,test
+       (list (cons ,car ,cdr)))))
+
 (declaim (ftype (sfunction ((or symbol cons) &optional (or null lexenv))
                            (values (member nil :function :macro :special-form)
                                    boolean
@@ -240,17 +245,16 @@ appear."
                 inlinep (info :function :inlinep name))))))
     (values binding
             localp
-            (let (alist)
-              (when (and ftype (neq *universal-fun-type* ftype))
-                (push (cons 'ftype (type-specifier ftype)) alist))
-              (ecase inlinep
-                ((:inline :maybe-inline) (push (cons 'inline 'inline) alist))
-                (:notinline (push (cons 'inline 'notinline) alist))
-                ((nil)))
-              (when dx (push (cons 'dynamic-extent t) alist))
-              (append alist (extra-pairs :function name fun *lexenv*))))))
-
-
+            (nconc (ecase inlinep
+                     ((:inline :maybe-inline)
+                      (list '(inline . inline)))
+                     (:notinline
+                      (list '(inline . notinline)))
+                     ((nil)))
+                   (list-cons-when (and ftype (neq *universal-fun-type* ftype))
+                     'ftype (type-specifier ftype))
+                   (list-cons-when dx 'dynamic-extent t)
+                   (extra-pairs :function name fun *lexenv*)))))
 
 (declaim (ftype (sfunction
                  (symbol &optional (or null lexenv))
@@ -358,14 +362,13 @@ appear."
                 localp nil))))
     (values binding
             localp
-            (let (alist)
-              (when ignorep (push (cons 'ignore t) alist))
-              (when (and type (neq *universal-type* type))
-                (push (cons 'type (type-specifier type)) alist))
-              (when dx (push (cons 'dynamic-extent t) alist))
-              (when (info :variable :always-bound name)
-                (push (cons 'sb-ext:always-bound t) alist))
-              (append alist (extra-pairs :variable name var *lexenv*))))))
+            (nconc (list-cons-when ignorep 'ignore t)
+                   (list-cons-when (and type (neq *universal-type* type))
+                     'type (type-specifier type))
+                   (list-cons-when dx 'dynamic-extent t)
+                   (list-cons-when (info :variable :always-bound name)
+                     'sb-ext:always-bound t)
+                   (extra-pairs :variable name var *lexenv*)))))
 
 (declaim (ftype (sfunction (symbol &optional (or null lexenv)) t)
                 declaration-information))
@@ -386,13 +389,10 @@ the condition types that have been muffled."
   (let ((env (or env (make-null-lexenv))))
     (case declaration-name
       (optimize
-       (let ((policy (sb-c::lexenv-policy env)))
-         (collect ((res))
-           (dolist (name sb-c::*policy-qualities*)
-             (res (list name (sb-c::policy-quality policy name))))
-           (loop for (name . nil) in sb-c::*policy-dependent-qualities*
-                 do (res (list name (sb-c::policy-quality policy name))))
-           (res))))
+       ;; CLtL2-mandated behavior:
+       ;; "The returned list always contains an entry for each of the standard
+       ;; qualities and for each of the implementation-specific qualities"
+       (sb-c::policy-to-decl-spec (sb-c::lexenv-policy env) nil t))
       (sb-ext:muffle-conditions
        (car (rassoc 'muffle-warning
                     (sb-c::lexenv-handled-conditions env))))
@@ -410,14 +410,8 @@ lambda expression will parse its form argument, binding the variables in
 LAMBDA-LIST appropriately, and then execute BODY with those bindings in
 effect."
   (declare (ignore env))
-  (with-unique-names (whole environment)
-    (multiple-value-bind (body decls)
-        (parse-defmacro lambda-list whole body name
-                        'parse-macro
-                        :environment environment)
-      `(lambda (,whole ,environment)
-         ,@decls
-         ,body))))
+  (make-macro-lambda (if (and name (symbolp name)) (string name) "PARSE-MACRO")
+                     lambda-list body 'parse-macro name))
 
 (defun enclose (lambda-expression &optional environment)
   "Return a function consistent with LAMBDA-EXPRESSION in ENVIRONMENT: the
