@@ -11,7 +11,7 @@
 
 (in-package "SB!KERNEL")
 
-(defvar *break-on-signals* nil
+(!defvar *break-on-signals* nil
   #!+sb-doc
   "When (TYPEP condition *BREAK-ON-SIGNALS*) is true, then calls to SIGNAL will
    enter the debugger prior to signalling that condition.")
@@ -83,15 +83,18 @@
         (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'signal)))
     (when *break-on-signals*
       (maybe-break-on-signal condition))
-    (loop
-      (unless handler-clusters
-        (return))
-      (let ((cluster (pop handler-clusters)))
+    (do ((cluster (pop handler-clusters) (pop handler-clusters)))
+        ((null cluster)
+         nil)
+      ;; Remove CLUSTER from *HANDLER-CLUSTERS*: if a condition is
+      ;; signaled in either the type test, i.e. (the function (car
+      ;; handler)), or the handler, (the function (cdr handler)), the
+      ;; recursive SIGNAL call should not consider CLUSTER as doing
+      ;; would lead to infinite recursive SIGNAL calls.
+      (let ((*handler-clusters* handler-clusters))
         (dolist (handler cluster)
-          (when (typep condition (car handler))
-            (let ((*handler-clusters* handler-clusters))
-              (funcall (cdr handler) condition))))))
-    nil))
+          (when (funcall (truly-the function (car handler)) condition)
+            (funcall (cdr handler) condition)))))))
 
 (defun error (datum &rest arguments)
   #!+sb-doc
@@ -156,17 +159,7 @@ of condition handling occurring."
    ARGUMENTS. While the condition is being signaled, a MUFFLE-WARNING restart
    exists that causes WARN to immediately return NIL."
   (/show0 "entering WARN")
-  ;; KLUDGE: The current cold load initialization logic causes several calls
-  ;; to WARN, so we need to be able to handle them without dying. (And calling
-  ;; FORMAT or even PRINC in cold load is a good way to die.) Of course, the
-  ;; ideal would be to clean up cold load so that it doesn't call WARN..
-  ;; -- WHN 19991009
-  (if (not *cold-init-complete-p*)
-      (progn
-        (/show0 "ignoring WARN in cold init, arguments=..")
-        #!+sb-show (dolist (argument arguments)
-                     (sb!impl::cold-print argument)))
-      (infinite-error-protect
+  (infinite-error-protect
        (/show0 "doing COERCE-TO-CONDITION")
        (let ((condition (coerce-to-condition datum arguments
                                              'simple-warning 'warn)))
@@ -187,5 +180,5 @@ of condition handling occurring."
                    "~&~@<~S: ~3i~:_~A~:>~%"
                    badness
                    condition)
-           (/show0 "back from FORMAT, voila!")))))
+           (/show0 "back from FORMAT, voila!"))))
   nil)
