@@ -172,7 +172,6 @@
   ;; the basic type machinery needs to be initialized before toplevel
   ;; forms run.
   (show-and-call !type-class-cold-init)
-  (show-and-call !typedefs-cold-init)
   (show-and-call !world-lock-cold-init)
   (show-and-call !classes-cold-init)
   (show-and-call !early-type-cold-init)
@@ -195,10 +194,10 @@
   (show-and-call sb!kernel::!set-up-structure-object-class)
 
   (dolist (x (nreverse *!reversed-cold-setf-macros*))
-    (apply #'!quietly-assign-setf-macro x))
+    (apply #'!quietly-defsetf x))
   (dolist (x (nreverse *!reversed-cold-defuns*))
-    (destructuring-bind (name &optional docstring . inline-expansion) x
-      (!%quietly-defun name docstring inline-expansion)))
+    (destructuring-bind (name . inline-expansion) x
+      (!%quietly-defun name inline-expansion)))
 
   ;; KLUDGE: Why are fixups mixed up with toplevel forms? Couldn't
   ;; fixups be done separately? Wouldn't that be clearer and better?
