@@ -165,64 +165,6 @@
               (push `(:include ,def!struct-supertype) options)))
         (values name `((,name ,@options) ,@rest) mlff def!struct-supertype)))))
 
-;;; Part of the raison d'etre for DEF!STRUCT is to be able to emulate
-;;; these low-level CMU CL functions in a vanilla ANSI Common Lisp
-;;; cross compilation host. (The emulation doesn't need to be
-;;; efficient, since it's needed for things like dumping objects, not
-;;; inner loops.)
-#+sb-xc-host
-(progn
-  (defun xc-dumpable-structure-instance-p (x)
-    (and (typep x 'cl:structure-object)
-         (let ((name (type-of x)))
-           ;; Don't allow totally random structures, only ones that the
-           ;; cross-compiler has been advised will work.
-           (and (get name :sb-xc-allow-dumping-instances)
-                ;; but we must also have cross-compiled it for real.
-                (sb!kernel::compiler-layout-ready-p name)
-                ;; and I don't know anything about raw slots
-                (zerop (layout-n-untagged-slots
-                        (info :type :compiler-layout name)))))))
-  (defun %instance-length (instance)
-    (aver (or (typep instance 'structure!object)
-              (xc-dumpable-structure-instance-p instance)))
-    ;; This seems strangely incompatible- INSTANCE-LENGTH should return an odd number
-    ;; so that including the header word (not counted), it comes out even.
-    ;; Except that on the xc host you can't access a slot beyond the layout-length,
-    ;; whereas on the target you can if there is a slack word - it reads as NIL.
-    (layout-length (classoid-layout (find-classoid (type-of instance)))))
-  (defun %instance-ref (instance index)
-    (aver (or (typep instance 'structure!object)
-              (xc-dumpable-structure-instance-p instance)))
-    (let* ((class (find-classoid (type-of instance)))
-           (layout (classoid-layout class)))
-      (if (zerop index)
-          layout
-          (let* ((dd (layout-info layout))
-                 (dsd (elt (dd-slots dd) (1- index)))
-                 (accessor-name (dsd-accessor-name dsd)))
-            (declare (type symbol accessor-name))
-            (funcall accessor-name instance)))))
-  ;; I believe this approach is technically nonportable because CLHS says that
-  ;;  "The mechanism by which defstruct arranges for slot accessors to be usable
-  ;;   with setf is implementation-dependent; for example, it may use setf
-  ;;   functions, setf expanders, or some other implementation-dependent
-  ;;   mechanism ..."
-  ;; As it happens, many implementations provide both functions and expanders.
-  (defun %instance-set (instance index new-value)
-    (aver (typep instance 'structure!object))
-    (let* ((class (find-classoid (type-of instance)))
-           (layout (classoid-layout class)))
-      (if (zerop index)
-          (error "can't set %INSTANCE-REF FOO 0 in cross-compilation host")
-          (let* ((dd (layout-info layout))
-                 (dsd (elt (dd-slots dd) (1- index)))
-                 (accessor-name (dsd-accessor-name dsd)))
-            (declare (type symbol accessor-name))
-            (funcall (fdefinition `(setf ,accessor-name))
-                     new-value
-                     instance))))))
-
 ;;; a helper function for DEF!STRUCT in the #+SB-XC-HOST case: Return
 ;;; DEFSTRUCT-style arguments with any class names in the SB!XC
 ;;; package (i.e. the name of the class being defined, and/or the
@@ -323,7 +265,7 @@
 ;;; STRUCTURE-OBJECTs defined by ordinary, post-warm-init programs, so
 ;;; it's only put into STRUCTURE-OBJECTs which inherit from
 ;;; STRUCTURE!OBJECT.)
-(def!struct (structure!object (:constructor nil)))
+(def!struct (structure!object (:constructor nil) (:copier nil) (:predicate nil)))
 
 ;;;; hooking this all into the standard MAKE-LOAD-FORM system
 
