@@ -66,8 +66,8 @@
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun %cxr-setf-expander (sub-accessor setter)
   (flet ((expand (place-reader original-form)
-           (let ((temp (sb!xc:gensym "LIST"))
-                 (newval (sb!xc:gensym "NEW")))
+           (let ((temp (make-symbol "LIST"))
+                 (newval (make-symbol "NEW")))
              (values (list temp)
                      `((,@place-reader ,@(cdr original-form)))
                      (list newval)
