@@ -109,67 +109,7 @@
 
 ;;; The DEFSTRUCT-DESCRIPTION structure holds compile-time information
 ;;; about a structure type.
-(def!struct (defstruct-description
-             (:conc-name dd-)
-             (:make-load-form-fun just-dump-it-normally)
-             #-sb-xc-host (:pure t)
-             (:constructor make-defstruct-description (name)))
-  ;; name of the structure
-  (name (missing-arg) :type symbol :read-only t)
-  ;; documentation on the structure
-  (doc nil :type (or string null))
-  ;; prefix for slot names. If NIL, none.
-  (conc-name nil :type (or string null))
-  ;; the name of the primary standard keyword constructor, or NIL if none
-  (default-constructor nil :type symbol)
-  ;; all the explicit :CONSTRUCTOR specs, with name defaulted
-  (constructors () :type list)
-  ;; name of copying function
-  (copier-name nil :type symbol)
-  ;; name of type predicate
-  (predicate-name nil :type symbol)
-  ;; the arguments to the :INCLUDE option, or NIL if no included
-  ;; structure
-  (include nil :type list)
-  ;; properties used to define structure-like classes with an
-  ;; arbitrary superclass and that may not have STRUCTURE-CLASS as the
-  ;; metaclass. Syntax is:
-  ;;    (superclass-name metaclass-name metaclass-constructor)
-  (alternate-metaclass nil :type list)
-  ;; a list of DEFSTRUCT-SLOT-DESCRIPTION objects for all slots
-  ;; (including included ones)
-  (slots () :type list)
-  ;; a list of (NAME . INDEX) pairs for accessors of included structures
-  (inherited-accessor-alist () :type list)
-  ;; number of elements we've allocated (See also RAW-LENGTH, which is not
-  ;; included in LENGTH.)
-  (length 0 :type index)
-  ;; General kind of implementation.
-  (type 'structure :type (member structure vector list
-                                 funcallable-structure))
-
-  ;; The next three slots are for :TYPE'd structures (which aren't
-  ;; classes, DD-CLASS-P = NIL)
-  ;;
-  ;; vector element type
-  (element-type t)
-  ;; T if :NAMED was explicitly specified, NIL otherwise
-  (named nil :type boolean)
-  ;; any INITIAL-OFFSET option on this direct type
-  (offset nil :type (or index null))
-
-  ;; which :PRINT-mumble option was given, if either was.
-  (print-option nil :type (member nil :print-function :print-object))
-  ;; the argument to the PRINT-FUNCTION or PRINT-OBJECT option.
-  ;; NIL if the option was given with no argument.
-  (printer-fname nil :type (or cons symbol))
-
-  ;; The number of untagged slots at the end.
-  #!-interleaved-raw-slots (raw-length 0 :type index)
-  ;; the value of the :PURE option, or :UNSPECIFIED. This is only
-  ;; meaningful if DD-CLASS-P = T.
-  (pure :unspecified :type (member t nil :unspecified)))
-#!-sb-fluid (declaim (freeze-type defstruct-description))
+;;; Its definition occurs in 'early-classoid.lisp'
 (def!method print-object ((x defstruct-description) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (prin1 (dd-name x) stream)))
@@ -251,6 +191,34 @@
       forms
       `(eval ',forms)))
 
+(defun %defstruct-package-locks (dd)
+  (let ((name (dd-name dd)))
+    (with-single-package-locked-error
+        (:symbol name "definining ~S as a structure"))
+    (when (dd-predicate-name dd)
+      (with-single-package-locked-error
+          (:symbol (dd-predicate-name dd)
+                   "defining ~s as a predicate for ~s structure"
+                   name)))
+
+    (when (dd-copier-name dd)
+      (with-single-package-locked-error
+          (:symbol (dd-copier-name dd)
+                   "defining ~s as a copier for ~s structure"
+                   name)))
+    (dolist (const (dd-constructors dd))
+      (when (car const)
+        (with-single-package-locked-error
+            (:symbol (car const)
+                     "defining ~s as a constructor for ~s structure"
+                     name))))
+    (dolist (dsd (dd-slots dd))
+      (when (dsd-accessor-name dsd)
+        (with-single-package-locked-error
+            (:symbol (dsd-accessor-name dsd)
+                     "defining ~s as an accessor for ~s structure"
+                     name))))))
+
 ;;; shared logic for host macroexpansion for SB!XC:DEFSTRUCT and
 ;;; cross-compiler macroexpansion for CL:DEFSTRUCT
 (defun %expander-for-defstruct (name-and-options slot-descriptions
@@ -286,21 +254,21 @@
                 (funcall #',fname ,x ,s
                          ,@(if depthp `(*current-level-in-print*)))))))))
     `(progn
-                ;; Note we intentionally enforce package locks and
-                ;; call %DEFSTRUCT first, and especially before
-                ;; %COMPILER-DEFSTRUCT. %DEFSTRUCT has the tests (and
-                ;; resulting CERROR) for collisions with LAYOUTs which
-                ;; already exist in the runtime. If there are any
-                ;; collisions, we want the user's response to CERROR
-                ;; to control what happens. Especially, if the user
-                ;; responds to the collision with ABORT, we don't want
-                ;; %COMPILER-DEFSTRUCT to modify the definition of the
-                ;; class.
-       ,@(when (eq expanding-into-code-for :target)
-           `((with-single-package-locked-error
-                 (:symbol ',name "defining ~A as a structure"))))
        ,@(if (dd-class-p dd)
-             `((%defstruct ',dd ',inherits (sb!c:source-location))
+             `(,@(when (eq expanding-into-code-for :target)
+                   `((eval-when (:compile-toplevel :load-toplevel :execute)
+                       ;; Note we intentionally enforce package locks and
+                       ;; call %DEFSTRUCT first, and especially before
+                       ;; %COMPILER-DEFSTRUCT. %DEFSTRUCT has the tests (and
+                       ;; resulting CERROR) for collisions with LAYOUTs which
+                       ;; already exist in the runtime. If there are any
+                       ;; collisions, we want the user's response to CERROR
+                       ;; to control what happens. Especially, if the user
+                       ;; responds to the collision with ABORT, we don't want
+                       ;; %COMPILER-DEFSTRUCT to modify the definition of the
+                       ;; class.
+                       (%defstruct-package-locks ',dd))))
+               (%defstruct ',dd ',inherits (sb!c:source-location))
                (eval-when (:compile-toplevel :load-toplevel :execute)
                  (%compiler-defstruct ',dd ',inherits))
                ,@(unless (eq expanding-into-code-for :host)
@@ -311,12 +279,6 @@
                              ,@(accessor-definitions dd)))
                 ;; This must be in the same lexical environment
                      ,@(constructor-definitions dd)
-                     ,@(when (eq (dd-pure dd) t)
-                         ;; Seems like %TARGET-DEFSTRUCT should do this
-                         `((locally
-                            (declare (notinline find-classoid))
-                            (setf (layout-pure (classoid-layout
-                                                (find-classoid ',name))) t))))
                      ,@print-method
                 ;; Various other operations only make sense on the target SBCL.
                      (%target-defstruct ',dd))))
@@ -1990,4 +1952,67 @@ or they must be declared locally notinline at each call site.~@:>")
          (defstruct-slot-description-p (cdr info))
          info)))
 
+;;; These functions are required to emulate SBCL kernel functions
+;;; in a vanilla ANSI Common Lisp cross-compilation host.
+;;; The emulation doesn't need to be efficient, since it's needed
+;;; only for object dumping.
+#+sb-xc-host
+(progn
+  (defun xc-dumpable-structure-instance-p (x)
+    (and (typep x 'cl:structure-object)
+         (let ((name (type-of x)))
+           ;; Don't allow totally random structures, only ones that the
+           ;; cross-compiler has been advised will work.
+           (and (get name :sb-xc-allow-dumping-instances)
+                ;; but we must also have cross-compiled it for real.
+                (sb!kernel::compiler-layout-ready-p name)
+                ;; and I don't know anything about raw slots
+                ;; Coincidentally, in either representation of
+                ;; raw-slot-metadata, 0 represents no untagged slots.
+                (zerop (layout-raw-slot-metadata
+                        (info :type :compiler-layout name)))))))
+  (defun %instance-layout (instance)
+    (aver (or (typep instance 'structure!object)
+              (xc-dumpable-structure-instance-p instance)))
+    (classoid-layout (find-classoid (type-of instance))))
+  (defun %instance-length (instance)
+    ;; INSTANCE-LENGTH tells you how many data words the backend is able to
+    ;; physically access in this structure. Since every structure occupies
+    ;; an even number of words, the storage slots comprise an odd number
+    ;; of words after subtracting 1 for the header.
+    ;; And in fact the fasl dumper / loader do write and read potentially
+    ;; one cell beyond the instance's LAYOUT-LENGTH if it was not odd.
+    ;; I'm not sure whether that is a good or bad thing.
+    ;; But be that as it may, in the cross-compiler you must not access
+    ;; more cells than there are in the declared structure because there
+    ;; is no lower level storage that you can peek at.
+    ;; So INSTANCE-LENGTH is exactly the same as LAYOUT-LENGTH on the host.
+    (layout-length (%instance-layout instance)))
+  (defun %instance-ref (instance index)
+    (let ((layout (%instance-layout instance)))
+      ;; with compact headers, 0 is an ordinary slot index.
+      ;; without, it's the layout.
+      (if (eql index (1- sb!vm:instance-data-start))
+          (error "XC Host should use %INSTANCE-LAYOUT, not %INSTANCE-REF 0")
+          (let* ((dd (layout-info layout))
+                 ;; If data starts at 1, then subtract 1 from index.
+                 ;; otherwise use the index as-is.
+                 (dsd (elt (dd-slots dd)
+                           (- index sb!vm:instance-data-start)))
+                 (accessor-name (dsd-accessor-name dsd)))
+            ;; Why AVER these: because it is slightly abstraction-breaking
+            ;; to assume that the slot-index N is the NTH item in the DSDs.
+            ;; The target Lisp never assumes that.
+            (aver (and (eql (dsd-index dsd) index) (eq (dsd-raw-type dsd) t)))
+            (funcall accessor-name instance)))))
+  ;; Setting with (FUNCALL `(SETF ,accessor) ...) is unportable because
+  ;;  "The mechanism by which defstruct arranges for slot accessors to be
+  ;;   usable with setf is implementation-dependent; for example, it may
+  ;;   use setf functions, setf expanders, or some other
+  ;;   implementation-dependent mechanism ..."
+  ;; But such capability seems not to be needed.
+  (defun %instance-set (instance index new-value)
+    (declare (ignore instance index new-value))
+    (error "Can not use %INSTANCE-SET on cross-compilation host.")))
+
 (/show0 "code/defstruct.lisp end of file")
