@@ -11,6 +11,14 @@
 
 (!begin-collecting-cold-init-forms)
 
+;;; the description of a &KEY argument
+(defstruct (key-info #-sb-xc-host (:pure t)
+                     (:copier nil))
+  ;; the key (not necessarily a keyword in ANSI Common Lisp)
+  (name (missing-arg) :type symbol :read-only t)
+  ;; the type of the argument value
+  (type (missing-arg) :type ctype :read-only t))
+
 ;;;; representations of types
 
 ;;; A HAIRY-TYPE represents anything too weird to be described
@@ -105,11 +113,16 @@
                                        (subseq optional 0 (1+ last-not-rest))))
                                 rest))))
 
-(defun parse-args-types (lambda-listy-thing)
+(defun parse-args-types (lambda-listy-thing context)
   (multiple-value-bind (llks required optional rest keys)
-      (parse-lambda-list lambda-listy-thing
-                         :context 'type :disallow '(&aux &environment)
-                         :silent t)
+      (parse-lambda-list
+       lambda-listy-thing
+       :context context
+       :accept (ecase context
+                 (:values-type (lambda-list-keyword-mask '(&optional &rest)))
+                 (:function-type (lambda-list-keyword-mask
+                                  '(&optional &rest &key &allow-other-keys))))
+       :silent t)
     (let ((required (mapcar #'single-value-specifier-type required))
           (optional (mapcar #'single-value-specifier-type optional))
           (rest (when rest (single-value-specifier-type (car rest))))
@@ -130,9 +143,7 @@
       (multiple-value-bind (required optional rest)
           (canonicalize-args-type-args required optional rest
                                        (ll-kwds-keyp llks))
-        (values required optional rest
-                (ll-kwds-keyp llks) keywords (ll-kwds-allowp llks)
-                (not (null llks)))))))
+        (values llks required optional rest keywords)))))
 
 (defstruct (values-type
             (:include args-type
@@ -447,6 +458,9 @@
 #!+sb-unicode
 (progn (defglobal *base-char-type* -1)
        (defglobal *extended-char-type* -1))
+#+sb-xc (declaim (type ctype *character-type*
+                       #!+sb-unicode *base-char-type*
+                       #!+sb-unicode *extended-char-type*))
 
 (defun !intern-important-character-set-type-instances ()
   (flet ((range (low high)
@@ -568,6 +582,10 @@
           (incf index 3))))
     (setq *rank-1-array-ctypes* (cons data-vector hashtable))))
 
+(declaim (ftype (sfunction (t &key (:complexp t)
+                                   (:element-type t)
+                                   (:specialized-element-type t))
+                           ctype) make-array-type))
 (defun make-array-type (dimensions &key (complexp :maybe) element-type
                                         (specialized-element-type *wild-type*))
   (or (and (eq element-type specialized-element-type)
@@ -578,10 +596,11 @@
                                   (mod (type-hash-value element-type)
                                        +canon-array-ctype-hash-divisor+)))
                (when (eq (car cell) element-type)
-                 (return (svref (car table)
+                 (return (truly-the ctype
+                          (svref (car table)
                                 (+ (cdr cell)
                                    (ecase complexp
-                                     ((nil) 0) ((:maybe) 1) ((t) 2)))))))))
+                                     ((nil) 0) ((:maybe) 1) ((t) 2))))))))))
       (%make-array-type dimensions
                         complexp element-type specialized-element-type)))
 
@@ -599,6 +618,7 @@
 (defglobal *null-type* -1)    ; = (MEMBER NIL)
 (defglobal *eql-t-type* -1)   ; = (MEMBER T)
 (defglobal *boolean-type* -1) ; = (MEMBER T NIL)
+#+sb-xc (declaim (type ctype *null-type*))
 
 (defun !intern-important-member-type-instances ()
   (flet ((make-it (list)
@@ -608,6 +628,8 @@
           *eql-t-type* (make-it '(t))
           *boolean-type* (make-it '(t nil)))))
 
+(declaim (ftype (sfunction (&key (:xset t) (:fp-zeroes t) (:members t)) ctype)
+                make-member-type))
 (defun make-member-type (&key xset fp-zeroes members)
   (unless xset
     (aver (not fp-zeroes))
@@ -767,12 +789,15 @@
 ;; The function caches work significantly better when there
 ;; is a unique object that stands for the specifier (CONS T T).
 (defglobal *cons-t-t-type* -1)
+#+sb-xc (declaim (type ctype *cons-t-t-type*))
 
 (defun !intern-important-cons-type-instances ()
   (setf *cons-t-t-type*
         (mark-ctype-interned
          (%make-cons-type *universal-type* *universal-type*))))
 
+#+sb-xc-host
+(declaim (ftype (sfunction (ctype ctype) (values t t)) type=))
 (defun make-cons-type (car-type cdr-type)
   (aver (not (or (eq car-type *wild-type*)
                  (eq cdr-type *wild-type*))))
@@ -787,23 +812,6 @@
         (t
          (%make-cons-type car-type cdr-type))))
 
-(defun cons-type-length-info (type)
-  (declare (type cons-type type))
-  (do ((min 1 (1+ min))
-       (cdr (cons-type-cdr-type type) (cons-type-cdr-type cdr)))
-      ((not (cons-type-p cdr))
-       (cond
-         ((csubtypep cdr (specifier-type 'null))
-          (values min t))
-         ((csubtypep *universal-type* cdr)
-          (values min nil))
-         ((type/= (type-intersection (specifier-type 'cons) cdr) *empty-type*)
-          (values min nil))
-         ((type/= (type-intersection (specifier-type 'null) cdr) *empty-type*)
-          (values min t))
-         (t (values min :maybe))))
-    ()))
-
 ;;; A SIMD-PACK-TYPE is used to represent a SIMD-PACK type.
 #!+sb-simd-pack
 (defstruct (simd-pack-type
@@ -949,65 +957,6 @@ expansion happened."
         (values (typexpand expansion env) t)
         (values expansion flag))))
 
-(defun typexpand-all (type-specifier &optional env)
-  #!+sb-doc
-  "Takes and expands a type specifier recursively like MACROEXPAND-ALL."
-  (declare (type type-specifier type-specifier))
-  (declare (ignore env))
-  ;; I first thought this would not be a good implementation because
-  ;; it signals an error on e.g. (CONS 1 2) until I realized that
-  ;; walking and calling TYPEXPAND would also result in errors, and
-  ;; it actually makes sense.
-  ;;
-  ;; There's still a small problem in that
-  ;;   (TYPEXPAND-ALL '(CONS * FIXNUM)) => (CONS T FIXNUM)
-  ;; whereas walking+typexpand would result in (CONS * FIXNUM).
-  ;;
-  ;; Similiarly, (TYPEXPAND-ALL '(FUNCTION (&REST T) *)) => FUNCTION.
-  (type-specifier (values-specifier-type type-specifier)))
-
-(defun defined-type-name-p (name &optional env)
-  #!+sb-doc
-  "Returns T if NAME is known to name a type specifier, otherwise NIL."
-  (declare (symbol name))
-  (declare (ignore env))
-  (and (info :type :kind name) t))
-
-(defun valid-type-specifier-p (type-specifier &optional env)
-  #!+sb-doc
-  "Returns T if TYPE-SPECIFIER is a valid type specifier, otherwise NIL.
-
-There may be different metrics on what constitutes a \"valid type
-specifier\" depending on context. If this function does not suit your
-exact need, you may be able to craft a particular solution using a
-combination of DEFINED-TYPE-NAME-P and the TYPEXPAND functions.
-
-The definition of \"valid type specifier\" employed by this function
-is based on the following mnemonic:
-
-          \"Would TYPEP accept it as second argument?\"
-
-Except that unlike TYPEP, this function fully supports compound
-FUNCTION type specifiers, and the VALUES type specifier, too.
-
-In particular, VALID-TYPE-SPECIFIER-P will return NIL if
-TYPE-SPECIFIER is not a class, not a symbol that is known to name a
-type specifier, and not a cons that represents a known compound type
-specifier in a syntactically and recursively correct way.
-
-Examples:
-
-  (valid-type-specifier-p '(cons * *))     => T
-  (valid-type-specifier-p '#:foo)          => NIL
-  (valid-type-specifier-p '(cons * #:foo)) => NIL
-  (valid-type-specifier-p '(cons 1 *)      => NIL
-
-Experimental."
-  (declare (ignore env))
-  (handler-case (prog1 t (values-specifier-type type-specifier))
-    (parse-unknown-type () nil)
-    (error () nil)))
-
 ;;; Note that the type NAME has been (re)defined, updating the
 ;;; undefined warnings and VALUES-SPECIFIER-TYPE cache.
 (defun %note-type-defined (name)
