@@ -255,28 +255,18 @@
 ;;;; alien type defining stuff
 
 (def!macro define-alien-type-translator (name lambda-list &body body)
-  (with-unique-names (whole env)
-    (let ((defun-name (symbolicate "ALIEN-" name "-TYPE-TRANSLATOR")))
-      (multiple-value-bind (body decls docs)
-          (sb!kernel:parse-defmacro lambda-list whole body name
-                                    'define-alien-type-translator
-                                    :environment env)
-        `(eval-when (:compile-toplevel :load-toplevel :execute)
-           (defun ,defun-name (,whole ,env)
-             (declare (ignorable ,env))
-             ,@decls
-             (block ,name
-               ,body))
-           (%define-alien-type-translator ',name #',defun-name ,docs))))))
+  (let ((defun-name (symbolicate "ALIEN-" name "-TYPE-TRANSLATOR")))
+    `(eval-when (:compile-toplevel :load-toplevel :execute)
+       (setf (symbol-function ',defun-name)
+             ,(make-macro-lambda defun-name lambda-list body
+                                 'define-alien-type-translator name))
+       (%define-alien-type-translator ',name #',defun-name))))
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-  (defun %define-alien-type-translator (name translator docs)
-    (declare (ignore docs))
+  (defun %define-alien-type-translator (name translator)
     (setf (info :alien-type :kind name) :primitive)
     (setf (info :alien-type :translator name) translator)
     (clear-info :alien-type :definition name)
-    #+nil
-    (setf (fdocumentation name 'alien-type) docs)
     name))
 
 (def!macro define-alien-type (name type &environment env)
