@@ -135,28 +135,6 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;;; routines to print objects
 
-
-;;; keyword variables shared by WRITE and WRITE-TO-STRING, and
-;;; the bindings they map to.
-(eval-when (:compile-toplevel :load-toplevel)
-  (defvar *printer-keyword-variables*
-    '(:escape *print-escape*
-      :radix *print-radix*
-      :base *print-base*
-      :circle *print-circle*
-      :pretty *print-pretty*
-      :level *print-level*
-      :length *print-length*
-      :case *print-case*
-      :array *print-array*
-      :gensym *print-gensym*
-      :readably *print-readably*
-      :right-margin *print-right-margin*
-      :miser-width *print-miser-width*
-      :lines *print-lines*
-      :pprint-dispatch *print-pprint-dispatch*
-      :suppress-errors *suppress-print-errors*)))
-
 (defun write (object &key
                      ((:stream stream) *standard-output*)
                      ((:escape *print-escape*) *print-escape*)
@@ -184,34 +162,6 @@ variable: an unreadable object representing the error is printed instead.")
   (output-object object (out-synonym-of stream))
   object)
 
-;;; Optimize common case of constant keyword arguments
-(define-compiler-macro write (&whole form object &rest keys)
-  (let (bind ignore)
-    (do ()
-        ((not (cdr keys))
-         ;; Odd number of keys, punt
-         (when keys
-           (return-from write form)))
-      (let* ((key (pop keys))
-             (value (pop keys))
-             (variable (or (getf *printer-keyword-variables* key)
-                           (when (eq :stream key)
-                             'stream)
-                           (return-from write form))))
-        (when (assoc variable bind)
-          ;; First key has precedence, but we still need to execute the
-          ;; argument, and in the right order.
-          (setf variable (gensym "IGNORE"))
-          (push variable ignore))
-        (push (list variable value) bind)))
-    (unless (assoc 'stream bind)
-      (push (list 'stream '*standard-output*) bind))
-    (once-only ((object object))
-      `(let ,(nreverse bind)
-         ,@(when ignore `((declare (ignore ,@ignore))))
-         (output-object ,object (out-synonym-of stream))
-         ,object))))
-
 (defun prin1 (object &optional stream)
   #!+sb-doc
   "Output a mostly READable printed representation of OBJECT on the specified
@@ -273,31 +223,6 @@ variable: an unreadable object representing the error is printed instead.")
   "Return the printed representation of OBJECT as a string."
   (stringify-object object))
 
-;;; Optimize common case of constant keyword arguments
-(define-compiler-macro write-to-string (&whole form object &rest keys)
-  (let (bind ignore)
-    (do ()
-        ((not (cdr keys))
-         ;; Odd number of keys, punt
-         (when keys
-           (return-from write-to-string form)))
-      (let* ((key (pop keys))
-             (value (pop keys))
-             (variable (or (getf *printer-keyword-variables* key)
-                           (return-from write-to-string form))))
-        (when (assoc variable bind)
-          ;; First key has precedence, but we still need to execute the
-          ;; argument, and in the right order.
-          (setf variable (gensym "IGNORE"))
-          (push variable ignore))
-        (push (list variable value) bind)))
-    (if bind
-        (once-only ((object object))
-          `(let ,(nreverse bind)
-             ,@(when ignore `((declare (ignore ,@ignore))))
-             (stringify-object ,object)))
-        `(stringify-object ,object))))
-
 (defun prin1-to-string (object)
   #!+sb-doc
   "Return the printed representation of OBJECT as a string with
@@ -549,38 +474,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; This variable contains the current definition of one of three
 ;;; symbol printers. SETUP-PRINTER-STATE sets this variable.
 (defvar *internal-symbol-output-fun* nil)
-
-;;; This function sets the internal global symbol
-;;; *INTERNAL-SYMBOL-OUTPUT-FUN* to the right function depending on
-;;; the value of *PRINT-CASE*. See the manual for details. The print
-;;; buffer stream is also reset.
-(defun setup-printer-state ()
-  (unless (and (eq *print-case* *previous-case*)
-               (eq (readtable-case *readtable*) *previous-readtable-case*))
-    (setq *previous-case* *print-case*)
-    (setq *previous-readtable-case* (readtable-case *readtable*))
-    (unless (member *print-case* '(:upcase :downcase :capitalize))
-      (setq *print-case* :upcase)
-      (error "invalid *PRINT-CASE* value: ~S" *previous-case*))
-    (unless (member *previous-readtable-case*
-                    '(:upcase :downcase :invert :preserve))
-      (setf (readtable-case *readtable*) :upcase)
-      (error "invalid READTABLE-CASE value: ~S" *previous-readtable-case*))
-
-    (setq *internal-symbol-output-fun*
-          (case *previous-readtable-case*
-            (:upcase
-             (case *print-case*
-               (:upcase #'output-preserve-symbol)
-               (:downcase #'output-lowercase-symbol)
-               (:capitalize #'output-capitalize-symbol)))
-            (:downcase
-             (case *print-case*
-               (:upcase #'output-uppercase-symbol)
-               (:downcase #'output-preserve-symbol)
-               (:capitalize #'output-capitalize-symbol)))
-            (:preserve #'output-preserve-symbol)
-            (:invert #'output-invert-symbol)))))
+(declaim (function *internal-symbol-output-fun*))
 
 ;;; Output PNAME (a symbol-name or package-name) surrounded with |'s,
 ;;; and with any embedded |'s or \'s escaped.
@@ -595,6 +489,7 @@ variable: an unreadable object representing the error is printed instead.")
   (write-char #\| stream))
 
 (defun output-symbol (object stream)
+  (declare (symbol object))
   (if (or *print-escape* *print-readably*)
       (let ((package (symbol-package object))
             (name (symbol-name object))
@@ -638,7 +533,10 @@ variable: an unreadable object representing the error is printed instead.")
   (declare (type simple-string name))
   (let ((*readtable* (if *print-readably* *standard-readtable* *readtable*)))
     (setup-printer-state)
-    (if (and maybe-quote (symbol-quotep name))
+    (if (and maybe-quote (or
+                          (and (readtable-normalization *readtable*)
+                               (not (sb!unicode:normalized-p name :nfkc)))
+                          (symbol-quotep name)))
         (output-quoted-symbol-name name stream)
         (funcall *internal-symbol-output-fun* name stream))))
 
@@ -771,7 +669,7 @@ variable: an unreadable object representing the error is printed instead.")
            (bases *digit-bases*)
            (base *print-base*)
            (letter-attribute
-            (case (readtable-case *readtable*)
+            (case (%readtable-case *readtable*)
               (:upcase uppercase-attribute)
               (:downcase lowercase-attribute)
               (t (logior lowercase-attribute uppercase-attribute))))
@@ -934,7 +832,7 @@ variable: an unreadable object representing the error is printed instead.")
 (defun output-capitalize-symbol (pname stream)
   (declare (simple-string pname))
   (let ((prev-not-alphanum t)
-        (up (eq (readtable-case *readtable*) :upcase)))
+        (up (eq (%readtable-case *readtable*) :upcase)))
     (dotimes (i (length pname))
       (let ((char (char pname i)))
         (write-char (if up
@@ -965,6 +863,33 @@ variable: an unreadable object representing the error is printed instead.")
           (t
            (write-string pname stream)))))
 
+;;; Set the internal global symbol *INTERNAL-SYMBOL-OUTPUT-FUN*
+;;; to the right function depending on the values of *PRINT-CASE*
+;;; and (%READTABLE-CASE *READTABLE*).
+(defun setup-printer-state ()
+  (let ((readtable-case (%readtable-case *readtable*))
+        (print-case *print-case*))
+    (unless (and (eq print-case *previous-case*)
+                 (eq readtable-case *previous-readtable-case*))
+      (setq *previous-case* print-case)
+      (setq *previous-readtable-case* readtable-case)
+      (setq *internal-symbol-output-fun*
+            ;; a morally equivalent reformulation of FOP-KNOWN-FUN
+            (macrolet ((load-time-fn (name) `(load-time-value #',name t)))
+             (case readtable-case
+               (:upcase
+                (case print-case
+                  (:upcase (load-time-fn output-preserve-symbol))
+                  (:downcase (load-time-fn output-lowercase-symbol))
+                  (:capitalize (load-time-fn output-capitalize-symbol))))
+               (:downcase
+                (case print-case
+                  (:upcase (load-time-fn output-uppercase-symbol))
+                  (:downcase (load-time-fn output-preserve-symbol))
+                  (:capitalize (load-time-fn output-capitalize-symbol))))
+               (:preserve (load-time-fn output-preserve-symbol))
+               (:invert (load-time-fn output-invert-symbol))))))))
+
 #|
 (defun test1 ()
   (let ((*readtable* (copy-readtable nil)))
@@ -1751,7 +1676,7 @@ variable: an unreadable object representing the error is printed instead.")
                            (standard-char-p char)))
             (name (char-name char)))
         (write-string "#\\" stream)
-        (if (and name (not graphicp))
+        (if (and name (or (not graphicp) *print-readably*))
             (quote-string name stream)
             (write-char char stream)))
       (write-char char stream)))
