@@ -141,7 +141,7 @@
 #!-sb-fluid (declaim (inline ansi-stream-file-position))
 (defun ansi-stream-file-position (stream position)
   (declare (type stream stream))
-  (declare (type (or index (alien sb!unix:off-t) (member nil :start :end))
+  (declare (type (or index (alien sb!unix:unix-offset) (member nil :start :end))
                  position))
   ;; FIXME: It would be good to comment on the stuff that is done here...
   ;; FIXME: This doesn't look interrupt safe.
@@ -235,61 +235,43 @@
 (defun ansi-stream-read-line-from-frc-buffer (stream eof-error-p eof-value)
   (prepare-for-fast-read-char stream
     (declare (ignore %frc-method%))
+    (declare (type ansi-stream-cin-buffer %frc-buffer%))
     (let ((chunks-total-length 0)
           (chunks nil))
       (declare (type index chunks-total-length)
                (list chunks))
       (labels ((refill-buffer ()
-                 (prog1
-                     (fast-read-char-refill stream nil nil)
+                 (prog1 (fast-read-char-refill stream nil)
                    (setf %frc-index% (ansi-stream-in-index %frc-stream%))))
-               (newline-position ()
-                 (position #\Newline (the (simple-array character (*))
-                                       %frc-buffer%)
-                           :test #'char=
-                           :start %frc-index%))
-               (make-and-return-result-string (pos)
-                 (let* ((len (+ (- (or pos %frc-index%)
-                                   %frc-index%)
-                                chunks-total-length))
-                        (res (make-string len))
-                        (start 0))
-                   (declare (type index start))
-                   (when chunks
-                     (dolist (chunk (nreverse chunks))
-                       (declare (type (simple-array character) chunk))
-                       (replace res chunk :start1 start)
-                       (incf start (length chunk))))
-                   (unless (null pos)
+               (build-result (pos n-more-chars)
+                 (let ((res (make-string (+ chunks-total-length n-more-chars)))
+                       (start1 chunks-total-length))
+                   (declare (type index start1))
+                   (when (>= pos 0)
                      (replace res %frc-buffer%
-                              :start1 start
-                              :start2 %frc-index% :end2 pos)
+                              :start1 start1 :start2 %frc-index% :end2 pos)
                      (setf %frc-index% (1+ pos)))
                    (done-with-fast-read-char)
-                   (return-from ansi-stream-read-line-from-frc-buffer (values res (null pos)))))
-               (add-chunk ()
-                 (let* ((end (length %frc-buffer%))
-                        (len (- end %frc-index%))
-                        (chunk (make-string len)))
-                   (replace chunk %frc-buffer% :start2 %frc-index% :end2 end)
-                   (push chunk chunks)
-                   (incf chunks-total-length len)
-                   (when (refill-buffer)
-                     (make-and-return-result-string nil)))))
-        (declare (inline make-and-return-result-string
-                         refill-buffer))
-        (when (and (= %frc-index% +ansi-stream-in-buffer-length+)
-                   (refill-buffer))
+                   (dolist (chunk chunks res)
+                     (declare (type (simple-array character (*)) chunk))
+                     (decf start1 (length chunk))
+                     (replace res chunk :start1 start1)))))
+        (declare (inline refill-buffer))
+        (if (or (< %frc-index% +ansi-stream-in-buffer-length+) (refill-buffer))
+            (loop
+             (let ((pos (position #\Newline %frc-buffer%
+                                  :test #'char= :start %frc-index%)))
+               (when pos
+                 (return (values (build-result pos (- pos %frc-index%)) nil)))
+               (let ((chunk (subseq %frc-buffer% %frc-index%)))
+                 (incf chunks-total-length (length chunk))
+                 (push chunk chunks))
+               (unless (refill-buffer)
+                 (return (values (build-result -1 0) t)))))
           ;; EOF had been reached before we read anything
           ;; at all. Return the EOF value or signal the error.
-          (done-with-fast-read-char)
-          (return-from ansi-stream-read-line-from-frc-buffer
-            (values (eof-or-lose stream eof-error-p eof-value) t)))
-        (loop
-           (let ((pos (newline-position)))
-             (if pos
-                 (make-and-return-result-string pos)
-                 (add-chunk))))))))
+            (progn (done-with-fast-read-char)
+                   (eof-or-lose stream eof-error-p (values eof-value t))))))))
 
 #!-sb-fluid (declaim (inline ansi-stream-read-line))
 (defun ansi-stream-read-line (stream eof-error-p eof-value recursive-p)
@@ -372,7 +354,11 @@
     (when (minusp index) (error "nothing to unread"))
     (cond (buffer
            (setf (aref buffer index) character)
-           (setf (ansi-stream-in-index stream) index))
+           (setf (ansi-stream-in-index stream) index)
+           ;; Ugh. an ANSI-STREAM with a char buffer never gives a chance to
+           ;; the stream's misc routine to handle the UNREAD operation.
+           (when (ansi-stream-input-char-pos stream)
+             (decf (ansi-stream-input-char-pos stream))))
           (t
            (funcall (ansi-stream-misc stream) stream
                     :unread character)))))
@@ -478,8 +464,7 @@
   (declare (type ansi-stream stream)
            (type index numbytes start)
            (type (or (simple-array * (*)) system-area-pointer) buffer))
-  (let* ((stream (in-synonym-of stream ansi-stream))
-         (in-buffer (ansi-stream-in-buffer stream))
+  (let* ((in-buffer (ansi-stream-in-buffer stream))
          (index (ansi-stream-in-index stream))
          (num-buffered (- +ansi-stream-in-buffer-length+ index)))
     (declare (fixnum index num-buffered))
@@ -529,7 +514,13 @@
 ;;; the IN-BUFFER for text streams. There is definitely an IN-BUFFER,
 ;;; and hence must be an N-BIN method. It's also called by other stream
 ;;; functions which directly peek into the frc buffer.
-(defun fast-read-char-refill (stream eof-error-p eof-value)
+;;; If EOF is hit and EOF-ERROR-P is false, then return NIL,
+;;; otherwise return the new index into CIN-BUFFER.
+(defun fast-read-char-refill (stream eof-error-p)
+  (when (ansi-stream-input-char-pos stream)
+    ;; Characters between (ANSI-STREAM-IN-INDEX %FRC-STREAM%)
+    ;; and +ANSI-STREAM-IN-BUFFER-LENGTH+ have to be re-scanned.
+    (update-input-char-pos stream))
   (let* ((ibuf (ansi-stream-cin-buffer stream))
          (count (funcall (ansi-stream-n-bin stream)
                          stream
@@ -558,15 +549,22 @@
            (let* ((index (1- +ansi-stream-in-buffer-length+))
                   (value (funcall (ansi-stream-in stream) stream nil :eof)))
              (cond
+               ;; When not signaling an error, it is important that IN-INDEX
+               ;; be set to +ANSI-STREAM-IN-BUFFER-LENGTH+ here, even though
+               ;; DONE-WITH-FAST-READ-CHAR will do the same, thereby writing
+               ;; the caller's %FRC-INDEX% (= +ANSI-STREAM-IN-BUFFER-LENGTH+)
+               ;; into the slot. But because we've already bumped INPUT-CHAR-POS
+               ;; and scanned characters between the original %FRC-INDEX%
+               ;; and the buffer end (above), we must *not* do that again.
                ((eql value :eof)
                 ;; definitely EOF now
                 (setf (ansi-stream-in-index stream)
                       +ansi-stream-in-buffer-length+)
-                (values t (eof-or-lose stream eof-error-p eof-value)))
+                (eof-or-lose stream eof-error-p nil))
                ;; we resynced or were given something instead
                (t
                 (setf (aref ibuf index) value)
-                (values nil (setf (ansi-stream-in-index stream) index))))))
+                (setf (ansi-stream-in-index stream) index)))))
           (t
            (when (/= start +ansi-stream-in-buffer-extra+)
              (#.(let* ((n-character-array-bits
@@ -580,8 +578,7 @@
                 ibuf +ansi-stream-in-buffer-extra+
                 ibuf start
                 count))
-           (values nil
-                   (setf (ansi-stream-in-index stream) start))))))
+           (setf (ansi-stream-in-index stream) start)))))
 
 ;;; This is similar to FAST-READ-CHAR-REFILL, but we don't have to
 ;;; leave room for unreading.
@@ -649,13 +646,6 @@
   (declare (type stream-designator stream))
   (%write-string string stream start end))
 
-;;; A wrapper function for all those (MACROLET OUT-FUN) definitions,
-;;; which cannot deal with keyword arguments. %WRITE-STRING cannot
-;;; replace this, as this needs to deal with simple-strings as well.
-(declaim (inline write-string-no-key))
-(defun write-string-no-key (string stream start end)
-  (write-string string stream :start start :end end))
-
 (defun write-line (string &optional (stream *standard-output*)
                    &key (start 0) end)
   (declare (type string string))
@@ -756,18 +746,21 @@
                                        (sout #'broadcast-sout)
                                        (misc #'broadcast-misc))
                              (:constructor %make-broadcast-stream
-                                           (&rest streams))
-                             (:copier nil))
+                                           (streams))
+                             (:copier nil)
+                             (:predicate nil))
   ;; a list of all the streams we broadcast to
   (streams () :type list :read-only t))
 
+(declaim (freeze-type broadcast-stream))
+
 (defun make-broadcast-stream (&rest streams)
   (dolist (stream streams)
     (unless (output-stream-p stream)
       (error 'type-error
              :datum stream
              :expected-type '(satisfies output-stream-p))))
-  (apply #'%make-broadcast-stream streams))
+  (%make-broadcast-stream streams))
 
 (macrolet ((out-fun (name fun &rest args)
              `(defun ,name (stream ,@args)
@@ -775,7 +768,7 @@
                   (,fun ,(car args) stream ,@(cdr args))))))
   (out-fun broadcast-out write-char char)
   (out-fun broadcast-bout write-byte byte)
-  (out-fun broadcast-sout write-string-no-key string start end))
+  (out-fun broadcast-sout %write-string string start end))
 
 (defun broadcast-misc (stream operation &optional arg1 arg2)
   (let ((streams (broadcast-stream-streams stream)))
@@ -795,27 +788,23 @@
       (:charpos
        (dolist (stream streams 0)
          (let ((charpos (charpos stream)))
-           (if charpos (return charpos)))))
+           (when charpos
+             (return charpos)))))
       (:line-length
        (let ((min nil))
          (dolist (stream streams min)
            (let ((res (line-length stream)))
              (when res (setq min (if min (min res min) res)))))))
       (:element-type
-       #+nil ; old, arguably more logical, version
-       (let (res)
-         (dolist (stream streams (if (> (length res) 1) `(and ,@res) t))
-           (pushnew (stream-element-type stream) res :test #'equal)))
-       ;; ANSI-specified version (under System Class BROADCAST-STREAM)
-       (let ((res t))
-         (do ((streams streams (cdr streams)))
-             ((null streams) res)
-           (when (null (cdr streams))
-             (setq res (stream-element-type (car streams)))))))
+       (let ((last (last streams)))
+         (if last
+             (stream-element-type (car last))
+             t)))
       (:external-format
-       (let ((res :default))
-         (dolist (stream streams res)
-           (setq res (stream-external-format stream)))))
+       (let ((last (last streams)))
+         (if last
+             (stream-external-format (car last))
+             :default)))
       (:file-length
        (let ((last (last streams)))
          (if last
@@ -826,13 +815,15 @@
            (let ((res (or (eql arg1 :start) (eql arg1 0))))
              (dolist (stream streams res)
                (setq res (file-position stream arg1))))
-           (let ((res 0))
-             (dolist (stream streams res)
-               (setq res (file-position stream))))))
+           (let ((last (last streams)))
+             (if last
+                 (file-position (car last))
+                 0))))
       (:file-string-length
-       (let ((res 1))
-         (dolist (stream streams res)
-           (setq res (file-string-length stream arg1)))))
+       (let ((last (last streams)))
+         (if last
+             (file-string-length (car last) arg1)
+             1)))
       (:close
        (set-closed-flame stream))
       (t
@@ -858,6 +849,9 @@
                            (:copier nil))
   ;; This is the symbol, the value of which is the stream we are synonym to.
   (symbol nil :type symbol :read-only t))
+
+(declaim (freeze-type synonym-stream))
+
 (def!method print-object ((x synonym-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream ":SYMBOL ~S" (synonym-stream-symbol x))))
@@ -871,7 +865,7 @@
                   (,fun ,(car args) syn ,@(cdr args))))))
   (out-fun synonym-out write-char ch)
   (out-fun synonym-bout write-byte n)
-  (out-fun synonym-sout write-string-no-key string start end))
+  (out-fun synonym-sout %write-string string start end))
 
 ;;; For the input methods, we just call the corresponding function on the
 ;;; synonymed stream. These functions deal with getting input out of
@@ -914,9 +908,11 @@
                       (sout #'two-way-sout)
                       (misc #'two-way-misc))
             (:constructor %make-two-way-stream (input-stream output-stream))
-            (:copier nil))
+            (:copier nil)
+            (:predicate nil))
   (input-stream (missing-arg) :type stream :read-only t)
   (output-stream (missing-arg) :type stream :read-only t))
+
 (defprinter (two-way-stream) input-stream output-stream)
 
 (defun make-two-way-stream (input-stream output-stream)
@@ -934,7 +930,7 @@
     (error 'type-error
            :datum input-stream
            :expected-type '(satisfies input-stream-p)))
-  (funcall #'%make-two-way-stream input-stream output-stream))
+  (%make-two-way-stream input-stream output-stream))
 
 (macrolet ((out-fun (name fun &rest args)
              `(defun ,name (stream ,@args)
@@ -942,7 +938,7 @@
                   (,fun ,(car args) syn ,@(cdr args))))))
   (out-fun two-way-out write-char ch)
   (out-fun two-way-bout write-byte n)
-  (out-fun two-way-sout write-string-no-key string start end))
+  (out-fun two-way-sout %write-string string start end))
 
 (macrolet ((in-fun (name fun &rest args)
              `(defun ,name (stream ,@args)
@@ -992,10 +988,14 @@
                       (bin #'concatenated-bin)
                       (n-bin #'concatenated-n-bin)
                       (misc #'concatenated-misc))
-            (:constructor %make-concatenated-stream (&rest streams))
-            (:copier nil))
+            (:constructor %make-concatenated-stream (streams))
+            (:copier nil)
+            (:predicate nil))
   ;; The car of this is the substream we are reading from now.
   (streams nil :type list))
+
+(declaim (freeze-type concatenated-stream))
+
 (def!method print-object ((x concatenated-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream
@@ -1011,7 +1011,7 @@
       (error 'type-error
              :datum stream
              :expected-type '(satisfies input-stream-p))))
-  (apply #'%make-concatenated-stream streams))
+  (%make-concatenated-stream streams))
 
 (macrolet ((in-fun (name fun)
              `(defun ,name (stream eof-error-p eof-value)
@@ -1087,8 +1087,12 @@
                       (misc #'echo-misc)
                       (n-bin #'echo-n-bin))
             (:constructor %make-echo-stream (input-stream output-stream))
-            (:copier nil))
+            (:copier nil)
+            (:predicate nil))
   (unread-stuff nil :type boolean))
+
+(declaim (freeze-type echo-stream))
+
 (def!method print-object ((x echo-stream) stream)
   (print-unreadable-object (x stream :type t :identity t)
     (format stream
@@ -1109,7 +1113,7 @@
     (error 'type-error
            :datum input-stream
            :expected-type '(satisfies input-stream-p)))
-  (funcall #'%make-echo-stream input-stream output-stream))
+  (%make-echo-stream input-stream output-stream))
 
 (macrolet ((in-fun (name in-fun out-fun &rest args)
              `(defun ,name (stream ,@args)
@@ -1168,16 +1172,17 @@
 (defstruct (string-input-stream
              (:include ansi-stream
                        (in #'string-inch)
-                       (bin #'ill-bin)
-                       (n-bin #'ill-bin)
                        (misc #'string-in-misc))
-             (:constructor internal-make-string-input-stream
+             (:constructor %make-string-input-stream
                            (string current end))
-             (:copier nil))
+             (:copier nil)
+             (:predicate nil))
   (string (missing-arg) :type simple-string :read-only t)
   (current (missing-arg) :type index)
   (end (missing-arg) :type index))
 
+(declaim (freeze-type string-input-stream))
+
 (defun string-inch (stream eof-error-p eof-value)
   (declare (type string-input-stream stream))
   (let ((string (string-input-stream-string stream))
@@ -1259,18 +1264,17 @@
   ;; adjustable string but (- END START) is 100 characters. We should use
   ;; SUBSEQ instead of coercing the whole string. And if STRING is non-simple
   ;; but has element type CHARACTER, wouldn't it work to just use the
-  ;; underlying simple-string since INTERNAL-MAKE- accepts bounding indices
-  ;; that can be fudged to deal with any offset?
+  ;; underlying simple-string since %MAKE-STRING-INPUT-STREAM accepts bounding
+  ;; indices that can be fudged to deal with any offset?
   ;; And (for unicode builds) if the input is BASE-STRING, we should use
   ;; MAKE-ARRAY and REPLACE to coerce just the specified piece.
   (let* ((string (coerce string '(simple-array character (*)))))
     ;; Why WITH-ARRAY-DATA, since the array is already simple?
     ;; because it's a nice abstract way to check the START and END.
     (with-array-data ((string string) (start start) (end end))
-      (internal-make-string-input-stream
+      (%make-string-input-stream
        string ;; now simple
-       start
-       end))))
+       start end))))
 
 ;;;; STRING-OUTPUT-STREAM stuff
 ;;;;
@@ -1291,13 +1295,13 @@
                       (sout #'string-sout)
                       (misc #'string-out-misc))
             (:constructor make-string-output-stream
-                          (&key (element-type 'character)
-                           &aux (buffer
-                                 (make-string
-                                  *string-output-stream-buffer-initial-size*))))
-            (:copier nil))
+                          (&key (element-type 'character)))
+            (:copier nil)
+            (:predicate nil))
   ;; The string we throw stuff in.
-  (buffer (missing-arg) :type (simple-array character (*)))
+  (buffer (make-string
+           *string-output-stream-buffer-initial-size*)
+   :type (simple-array character (*)))
   ;; Chains of buffers to use
   (prev nil :type list)
   (next nil :type list)
@@ -1310,7 +1314,10 @@
   ;; end of the stream.
   (index-cache 0 :type index)
   ;; Requested element type
-  (element-type 'character :type type-specifier))
+  (element-type 'character :type type-specifier
+                           :read-only t))
+
+(declaim (freeze-type string-output-stream))
 
 #!+sb-doc
 (setf (fdocumentation 'make-string-output-stream 'function)
@@ -1594,10 +1601,13 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
                       (sout #'fill-pointer-sout)
                       (misc #'fill-pointer-misc))
             (:constructor make-fill-pointer-output-stream (string))
-            (:copier nil))
+            (:copier nil)
+            (:predicate nil))
   ;; a string with a fill pointer where we stuff the stuff we write
   (string (missing-arg) :type string-with-fill-pointer :read-only t))
 
+(declaim (freeze-type fill-pointer-output-stream))
+
 (defun fill-pointer-ouch (stream character)
   (let* ((buffer (fill-pointer-output-stream-string stream))
          (current (fill-pointer buffer))
@@ -1705,7 +1715,9 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
                       (misc #'case-frob-misc))
             (:constructor %make-case-frob-stream (target out sout))
             (:copier nil))
-  (target (missing-arg) :type stream))
+  (target (missing-arg) :type stream :read-only t))
+
+(declaim (freeze-type case-frob-stream))
 
 (defun make-case-frob-stream (target kind)
   #!+sb-doc
@@ -2035,8 +2047,7 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
       (unless %frc-buffer%
         (return-from ansi-stream-read-string-from-frc-buffer nil))
       (labels ((refill-buffer ()
-                 (prog1
-                     (fast-read-char-refill stream nil nil)
+                 (prog1 (fast-read-char-refill stream nil)
                    (setf %frc-index% (ansi-stream-in-index %frc-stream%))))
                (add-chunk ()
                  (let* ((end (length %frc-buffer%))
@@ -2053,16 +2064,15 @@ benefit of the function GET-OUTPUT-STREAM-STRING.")
                               :end2 (+ %frc-index% len)))
                    (incf read len)
                    (incf %frc-index% len)
-                   (when (or (eql needed read)
-                             (refill-buffer))
+                   (when (or (eql needed read) (not (refill-buffer)))
                      (done-with-fast-read-char)
                      (return-from ansi-stream-read-string-from-frc-buffer
                        (+ start read))))))
         (declare (inline refill-buffer))
         (when (and (= %frc-index% +ansi-stream-in-buffer-length+)
-                   (refill-buffer))
+                   (not (refill-buffer)))
           ;; EOF had been reached before we read anything
-          ;; at all. Return the EOF value or signal the error.
+          ;; at all. But READ-SEQUENCE never signals an EOF error.
           (done-with-fast-read-char)
           (return-from ansi-stream-read-string-from-frc-buffer start))
         (loop (add-chunk))))))
