@@ -16,17 +16,21 @@
   (declare (special *muffled-warnings*))
   (typep warning *muffled-warnings*))
 
-(defun initial-handler-clusters ()
-  `(((warning . ,#'(lambda (warning)
-                     (when (muffle-warning-p warning)
-                       (muffle-warning warning)))))))
+(defglobal **initial-handler-clusters**
+  `(((,(lambda (condition)
+         (typep condition 'warning))
+      .
+      ,(lambda (warning)
+         (when (muffle-warning-p warning)
+           (muffle-warning warning)))))))
 
 ;;; Each cluster is an alist of the form
 ;;;
-;;;  ((CONDITION-TYPE1 . HANDLER1) (CONDITION-TYPE2 . HANDLER2) ...)
+;;;  ((TYPE-TEST1 . HANDLER1) (TYPE-TEST2 . HANDLER2) ...)
 ;;;
-;;; where CONDITION-TYPEN are type specifiers and HANDLERN are
-;;; function designators.
+;;; where TYPE-TESTN are functions of one argument which test a given
+;;; condition instance for the type required by the corresponding
+;;; HANDLERN. HANDLERN are function designators.
 ;;;
 ;;; Newly established handlers are added at the beginning of the
 ;;; list. Elements to the left of the alist take precedence over
@@ -34,37 +38,12 @@
 ;;;
 ;;; Lists to which *HANDLER-CLUSTERS* is bound generally have dynamic
 ;;; extent.
-(defvar *handler-clusters* (initial-handler-clusters))
+(defvar *handler-clusters* **initial-handler-clusters**)
 
 ;;; a list of lists of currently active RESTART instances. maintained
 ;;; by RESTART-BIND.
 (defvar *restart-clusters* '())
 
-(declaim (inline restart-test-function
-                 restart-associated-conditions
-                 (setf restart-associated-conditions)))
-(defstruct (restart (:constructor make-restart
-                        ;; Having TEST-FUNCTION at the end allows
-                        ;; to not replicate its default value in RESTART-BIND.
-                        (name function
-                         &optional report-function
-                                   interactive-function
-                                   test-function))
-                    (:copier nil) (:predicate nil))
-  (name (missing-arg) :type symbol :read-only t)
-  (function (missing-arg) :type function :read-only t)
-  (report-function nil :type (or null function) :read-only t)
-  (interactive-function nil :type (or null function) :read-only t)
-  (test-function (lambda (cond) (declare (ignore cond)) t) :type function :read-only t)
-  ;; the list of conditions which are currently associated to the
-  ;; restart. maintained by WITH-CONDITION-RESTARTS in a neither
-  ;; thread- nor interrupt-safe way. This should not be a problem
-  ;; however, since safe uses of restarts have to assume dynamic
-  ;; extent.
-  (associated-conditions '() :type list))
-
-#!-sb-fluid (declaim (freeze-type restart))
-
 (def!method print-object ((restart restart) stream)
   (if *print-escape*
       (print-unreadable-object (restart stream :type t :identity t)
@@ -76,11 +55,12 @@
       "Return the name of the given restart object.")
 
 (defun restart-report (restart stream)
-  (funcall (or (restart-report-function restart)
-               (lambda (stream)
-                 (format stream "~S" (or (restart-name restart)
-                                         restart))))
-           stream))
+  (if (restart-report-function restart)
+      (funcall (truly-the function (restart-report-function restart))
+               stream)
+      (prin1 (or (restart-name restart)
+                 restart)
+             stream)))
 
 (defvar *restart-test-stack* nil)
 
