@@ -70,22 +70,27 @@
 
 (def!method print-object ((table package-hashtable) stream)
   (declare (type stream stream))
-  (print-unreadable-object (table stream :type t)
-    (format stream
-            ":SIZE ~S :FREE ~S :DELETED ~S"
-            (package-hashtable-size table)
-            (package-hashtable-free table)
-            (package-hashtable-deleted table))))
+  (print-unreadable-object (table stream :type t :identity t)
+    (let* ((n-live (%package-hashtable-symbol-count table))
+           (n-deleted (package-hashtable-deleted table))
+           (n-filled (+ n-live n-deleted))
+           (n-cells (1- (length (package-hashtable-cells table)))))
+      (format stream
+              "(~D+~D)/~D [~@[~,3f words/sym,~]load=~,1f%]"
+              n-live n-deleted n-cells
+              (unless (zerop n-live)
+                (/ (* (1+ (/ sb!vm:n-word-bytes)) n-cells) n-live))
+              (* 100 (/ n-filled n-cells))))))
 
 ;;; the maximum load factor we allow in a package hashtable
-(defconstant +package-rehash-threshold+ 0.75)
+(!defparameter *package-rehash-threshold* 3/4)
 
 ;;; the load factor desired for a package hashtable when writing a
 ;;; core image
-(defconstant +package-hashtable-image-load-factor+ 0.5)
+(!defparameter *package-hashtable-image-load-factor* 1/2)
 
 ;;; Make a package hashtable having a prime number of entries at least
-;;; as great as (/ SIZE +PACKAGE-REHASH-THRESHOLD+). If RES is supplied,
+;;; as great as (/ SIZE *PACKAGE-REHASH-THRESHOLD*). If RES is supplied,
 ;;; then it is destructively modified to produce the result. This is
 ;;; useful when changing the size, since there are many pointers to
 ;;; the hashtable.
@@ -95,11 +100,11 @@
 (defun make-package-hashtable (size)
   (flet ((actual-package-hashtable-size (size)
            (loop for n of-type fixnum
-              from (logior (ceiling size +package-rehash-threshold+) 1)
+              from (logior (ceiling size *package-rehash-threshold*) 1)
               by 2
               when (positive-primep n) return n)))
     (let* ((n (actual-package-hashtable-size size))
-           (size (truncate (* n +package-rehash-threshold+)))
+           (size (truncate (* n *package-rehash-threshold*)))
            (table (make-array (1+ n) :initial-element 0)))
       (setf (aref table n)
             (make-array n :element-type '(unsigned-byte 8)
@@ -109,14 +114,14 @@
 ;;; Destructively resize TABLE to have room for at least SIZE entries
 ;;; and rehash its existing entries.
 (defun resize-package-hashtable (table size)
-  (let* ((symvec (package-hashtable-table table))
+  (let* ((symvec (package-hashtable-cells table))
          (len (1- (length symvec)))
          (temp-table (make-package-hashtable size)))
     (dotimes (i len)
       (let ((sym (svref symvec i)))
-        (unless (eql sym 0)
+        (when (pkg-symbol-valid-p sym)
           (add-symbol temp-table sym))))
-    (setf (package-hashtable-table table) (package-hashtable-table temp-table)
+    (setf (package-hashtable-cells table) (package-hashtable-cells temp-table)
           (package-hashtable-size table) (package-hashtable-size temp-table)
           (package-hashtable-free table) (package-hashtable-free temp-table)
           (package-hashtable-deleted table) 0)))
@@ -572,12 +577,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; Return a list of packages given a package designator or list of
 ;;; package designators, or die trying.
 (defun package-listify (thing)
-  (mapcar #'find-undeleted-package-or-lose
-          (if (listp thing) thing (list thing))))
-
-;;; Make a package name into a simple-string.
-(defun package-namify (n)
-  (stringify-package-designator n))
+  (mapcar #'find-undeleted-package-or-lose (ensure-list thing)))
 
 ;;; ANSI specifies (in the definition of DELETE-PACKAGE) that PACKAGE-NAME
 ;;; returns NIL (not an error) for a deleted package, so this is a special
@@ -588,20 +588,14 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 
 ;;;; operations on package hashtables
 
-;;; Compute a number from the sxhash of the pname and the length which
-;;; must be between 2 and 255.
-(defmacro entry-hash (length sxhash)
-  `(the fixnum
-        (+ (the fixnum
-                (rem (the fixnum
-                          (logxor ,length
-                                  ,sxhash
-                                  (the fixnum (ash ,sxhash -8))
-                                  (the fixnum (ash ,sxhash -16))
-                                  (the fixnum (ash ,sxhash -19))))
-                     254))
-           2)))
-;;; FIXME: should be wrapped in EVAL-WHEN (COMPILE EXECUTE)
+;;; Compute a number between 1 and 255 based on the sxhash of the
+;;; pname and the length thereof.
+(declaim (inline entry-hash))
+(defun entry-hash (length sxhash)
+  (declare (index length) ((and fixnum unsigned-byte) sxhash))
+  (1+ (rem (logxor length sxhash
+                   (ash sxhash -8) (ash sxhash -16) (ash sxhash -19))
+           255)))
 
 ;;; Add a symbol to a package hashtable. The symbol is assumed
 ;;; not to be present.
@@ -615,35 +609,34 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                               (* (- (package-hashtable-size table)
                                     (package-hashtable-deleted table))
                                  2)))
-  (let* ((symvec (package-hashtable-table table))
+  (let* ((symvec (package-hashtable-cells table))
          (len (1- (length symvec)))
          (hashvec (the hash-vector (aref symvec len)))
-         (sxhash (%sxhash-simple-string (symbol-name symbol)))
+         (sxhash (truly-the fixnum (ensure-symbol-hash symbol)))
          (h2 (1+ (rem sxhash (- len 2)))))
     (declare (fixnum sxhash h2))
     (do ((i (rem sxhash len) (rem (+ i h2) len)))
-        ((< (aref hashvec i) 2)
-         (if (eql (aref hashvec i) 0)
+        ((eql (aref hashvec i) 0)
+         (if (eql (svref symvec i) 0)
              (decf (package-hashtable-free table))
              (decf (package-hashtable-deleted table)))
          ;; This order of these two SETFs does not matter.
-         ;; A symbol cell of 0 is skipped on lookup if the hash cell
-         ;; matches something accidentally.
+         ;; An empty symbol cell is skipped on lookup if the hash cell
+         ;; matches something accidentally. "empty" = any fixnum.
          (setf (svref symvec i) symbol)
          (setf (aref hashvec i)
-               (entry-hash (length (symbol-name symbol))
-                           sxhash)))
+               (entry-hash (length (symbol-name symbol)) sxhash)))
       (declare (fixnum i)))))
 
 ;;; Resize the package hashtables of all packages so that their load
-;;; factor is +PACKAGE-HASHTABLE-IMAGE-LOAD-FACTOR+. Called from
+;;; factor is *PACKAGE-HASHTABLE-IMAGE-LOAD-FACTOR*. Called from
 ;;; SAVE-LISP-AND-DIE to optimize space usage in the image.
 (defun tune-hashtable-sizes-of-all-packages ()
   (flet ((tune-table-size (table)
            (resize-package-hashtable
             table
-            (round (* (/ +package-rehash-threshold+
-                         +package-hashtable-image-load-factor+)
+            (round (* (/ *package-rehash-threshold*
+                         *package-hashtable-image-load-factor*)
                       (- (package-hashtable-size table)
                          (package-hashtable-free table)
                          (package-hashtable-deleted table)))))))
@@ -659,9 +652,8 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 
 (defmacro with-symbol (((symbol-var &optional (index-var (gensym))) table
                        string length sxhash entry-hash) &body forms)
-  (let ((vec (gensym)) (hash-vec (gensym)) (len (gensym)) (h2 (gensym))
-        (name (gensym)) (ehash (gensym)))
-    `(let* ((,vec (package-hashtable-table ,table))
+  (with-unique-names (vec len hash-vec h2 probed-ehash name)
+    `(let* ((,vec (package-hashtable-cells ,table))
             (,len (1- (length ,vec)))
             (,hash-vec (the hash-vector (svref ,vec ,len)))
             (,index-var (rem (the hash ,sxhash) ,len))
@@ -669,23 +661,28 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                                       (the index (- ,len 2)))))))
        (declare (type index ,len ,h2 ,index-var))
        (loop
-        (let ((,ehash (aref ,hash-vec ,index-var)))
-          (cond ((eql ,ehash ,entry-hash)
-                 (let ((,symbol-var (svref ,vec ,index-var)))
-                   (unless (eql ,symbol-var 0)
-                     (let ((,name (symbol-name (truly-the symbol ,symbol-var))))
-                       (when (and (= (length ,name) ,length)
-                                  (string= ,string ,name
-                                           :end1 ,length :end2 ,length))
-                         (return (progn ,@forms)))))))
-                 ((zerop ,ehash)
-                  (return))))
-        (setq ,index-var (+ ,index-var ,h2))
-        (when (>= ,index-var ,len)
-          (setq ,index-var (- ,index-var ,len)))))))
+        (let ((,probed-ehash (aref ,hash-vec ,index-var)))
+          (cond
+           ((eql ,probed-ehash ,entry-hash)
+            (let ((,symbol-var (truly-the symbol (svref ,vec ,index-var))))
+              (when (eq (symbol-hash ,symbol-var) ,sxhash)
+                (let ((,name (symbol-name ,symbol-var)))
+                  ;; The pre-test for length is kind of an unimportant
+                  ;; optimization, but passing it for both :end arguments
+                  ;; requires that it be within bounds for the probed symbol.
+                  (when (and (= (length ,name) ,length)
+                             (string= ,string ,name
+                                      :end1 ,length :end2 ,length))
+                    (return (progn ,@forms)))))))
+           ((eql ,probed-ehash 0)
+            ;; either a never used cell or a tombstone left by UNINTERN
+            (when (eql (svref ,vec ,index-var) 0) ; really never used
+              (return)))))
+        (when (>= (incf ,index-var ,h2) ,len)
+          (decf ,index-var ,len))))))
 
 ;;; Delete the entry for STRING in TABLE. The entry must exist.
-;;; Deletion stores 0 for the symbol and 1 for the hash tombstone.
+;;; Deletion stores -1 for the symbol and 0 for the hash tombstone.
 ;;; Storing NIL for the symbol, as used to be done, is vulnerable to a rare
 ;;; concurrency bug because many strings have the same ENTRY-HASH as NIL:
 ;;;  (entry-hash 3 (sxhash "NIL")) => 177 and
@@ -708,20 +705,20 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; check is required, since a symbol is both its key and value,
 ;;; and the "absence of a symbol" marker is never mistaken for a symbol.
 ;;;
-(defun nuke-symbol (table string)
-  (declare (simple-string string))
-  (let* ((length (length string))
-         (hash (%sxhash-simple-string string))
+(defun nuke-symbol (table symbol)
+  (let* ((string (symbol-name symbol))
+         (length (length string))
+         (hash (symbol-hash symbol))
          (ehash (entry-hash length hash)))
     (declare (type index length)
              (type hash hash))
     (with-symbol ((symbol index) table string length hash ehash)
       ;; It is suboptimal to grab the vectors again, but not broken,
       ;; because we have exclusive use of the table for writing.
-      (let* ((symvec (package-hashtable-table table))
+      (let* ((symvec (package-hashtable-cells table))
              (hashvec (the hash-vector (aref symvec (1- (length symvec))))))
-        (setf (aref hashvec index) 1)
-        (setf (aref symvec index) 0))
+        (setf (aref hashvec index) 0)
+        (setf (aref symvec index) -1)) ; any nonzero fixnum will do
       (incf (package-hashtable-deleted table))))
   ;; If the table is less than one quarter full, halve its size and
   ;; rehash the entries.
@@ -740,7 +737,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 (defun %enter-new-nicknames (package nicknames)
   (declare (type list nicknames))
   (dolist (n nicknames)
-    (let* ((n (package-namify n))
+    (let* ((n (stringify-package-designator n))
            (found (with-package-names (names)
                     (or (gethash n names)
                         (progn
@@ -786,12 +783,12 @@ implementation it is ~S." *default-package-use-list*)
        ;; Check for race, signal the error outside the lock.
        (when (and (not clobber) (find-package name))
          (go :restart))
-       (let* ((name (package-namify name))
+       (let* ((name (stringify-package-designator name))
               (package
-               (internal-make-package
-                :%name name
-                :internal-symbols (make-package-hashtable internal-symbols)
-                :external-symbols (make-package-hashtable external-symbols))))
+               (%make-package
+                name
+                (make-package-hashtable internal-symbols)
+                (make-package-hashtable external-symbols))))
 
          ;; Do a USE-PACKAGE for each thing in the USE list so that checking for
          ;; conflicting exports among used packages is done.
@@ -822,7 +819,7 @@ implementation it is ~S." *default-package-use-list*)
   "Changes the name and nicknames for a package."
   (prog () :restart
      (let ((package (find-undeleted-package-or-lose package-designator))
-           (name (package-namify name))
+           (name (stringify-package-designator name))
            (found (find-package name))
            (nicks (mapcar #'string nicknames)))
        (unless (or (not found) (eq found package))
@@ -930,7 +927,7 @@ implementation it is ~S." *default-package-use-list*)
     (with-package-names (names)
       (maphash (lambda (k v)
                  (declare (ignore k))
-                 (pushnew v res))
+                 (pushnew v res :test #'eq))
                names))
     res))
 
@@ -980,7 +977,7 @@ implementation it is ~S." *default-package-use-list*)
 ;;; If the symbol named by the first LENGTH characters of NAME doesn't exist,
 ;;; then create it, special-casing the keyword package.
 (defun intern* (name length package &key no-copy)
-  (declare (simple-string name))
+  (declare (simple-string name) (index length))
   (multiple-value-bind (symbol where) (find-symbol* name length package)
     (cond (where
            (values symbol where))
@@ -1010,13 +1007,13 @@ implementation it is ~S." *default-package-use-list*)
                    (with-single-package-locked-error
                        (:package package "interning ~A" symbol-name)
                      (let ((symbol (make-symbol symbol-name)))
+                       (add-symbol (cond ((eq package *keyword-package*)
+                                          (%set-symbol-value symbol symbol)
+                                          (package-external-symbols package))
+                                         (t
+                                          (package-internal-symbols package)))
+                                   symbol)
                        (%set-symbol-package symbol package)
-                       (cond
-                         ((eq package *keyword-package*)
-                          (%set-symbol-value symbol symbol)
-                          (add-symbol (package-external-symbols package) symbol))
-                         (t
-                          (add-symbol (package-internal-symbols package) symbol)))
                        (values symbol nil))))))))))
 
 ;;; Check internal and external symbols, then scan down the list
@@ -1024,7 +1021,7 @@ implementation it is ~S." *default-package-use-list*)
 (defun find-symbol* (string length package)
   (declare (simple-string string)
            (type index length))
-  (let* ((hash (%sxhash-simple-substring string length))
+  (let* ((hash (compute-symbol-hash string length))
          (ehash (entry-hash length hash)))
     (declare (type hash hash ehash))
     (with-symbol ((symbol) (package-internal-symbols package)
@@ -1055,10 +1052,12 @@ implementation it is ~S." *default-package-use-list*)
 ;;; Return the symbol and T if found, otherwise two NILs.
 ;;; This is used for fast name-conflict checking in this file and symbol
 ;;; printing in the printer.
+;;; An optimization is possible here: by accepting either a string or symbol,
+;;; if the symbol's hash slot is nonzero, we can avoid COMPUTE-SYMBOL-HASH.
 (defun find-external-symbol (string package)
   (declare (simple-string string))
   (let* ((length (length string))
-         (hash (%sxhash-simple-string string))
+         (hash (compute-symbol-hash string length))
          (ehash (entry-hash length hash)))
     (declare (type index length)
              (type hash hash))
@@ -1067,17 +1066,6 @@ implementation it is ~S." *default-package-use-list*)
       (return-from find-external-symbol (values symbol t))))
   (values nil nil))
 
-(defun print-symbol-with-prefix (stream symbol colon at)
-  #!+sb-doc
-  "For use with ~/: Write SYMBOL to STREAM as if it is not accessible from
-  the current package."
-  (declare (ignore colon at))
-  ;; Only keywords should be accessible from the keyword package, and
-  ;; keywords are always printed with colons, so this guarantees that the
-  ;; symbol will not be printed without a prefix.
-  (let ((*package* *keyword-package*))
-    (write symbol :stream stream :escape t)))
-
 (define-condition name-conflict (reference-condition package-error)
   ((function :initarg :function :reader name-conflict-function)
    (datum :initarg :datum :reader name-conflict-datum)
@@ -1219,7 +1207,9 @@ uninterned."
           (let ((cset ()))
             (dolist (p (package-%use-list package))
               (multiple-value-bind (s w) (find-external-symbol name p)
-                (when w (pushnew s cset))))
+                ;; S should be derived as SYMBOL so that PUSHNEW can assume #'EQ
+                ;; as the test, but it's not happening, so restate the obvious.
+                (when w (pushnew s cset :test #'eq))))
             (when (cdr cset)
               (apply #'name-conflict package 'unintern symbol cset)
               (return-from unintern t)))
@@ -1232,7 +1222,7 @@ uninterned."
                  (nuke-symbol (if (eq w :internal)
                                   (package-internal-symbols package)
                                   (package-external-symbols package))
-                              name)
+                              symbol)
                  (if (eq (symbol-package symbol) package)
                      (%set-symbol-package symbol nil))
                  t)
@@ -1253,9 +1243,7 @@ uninterned."
                                thing))))
 
 (defun string-listify (thing)
-  (mapcar #'string (if (listp thing)
-                       thing
-                       (list thing))))
+  (mapcar #'string (ensure-list thing)))
 
 (defun export (symbols &optional (package (sane-package)))
   #!+sb-doc
@@ -1310,7 +1298,7 @@ uninterned."
               (external (package-external-symbols package)))
           (dolist (sym syms)
             (add-symbol external sym)
-            (nuke-symbol internal (symbol-name sym)))))
+            (nuke-symbol internal sym))))
       t)))
 
 ;;; Check that all symbols are accessible, then move from external to internal.
@@ -1337,7 +1325,7 @@ uninterned."
               (external (package-external-symbols package)))
           (dolist (sym syms)
             (add-symbol internal sym)
-            (nuke-symbol external (symbol-name sym)))))
+            (nuke-symbol external sym))))
       t)))
 
 ;;; Check for name conflict caused by the import and let the user
@@ -1558,17 +1546,18 @@ PACKAGE."
             (string (stringify-string-designator string-designator))
             (result nil))
         (do-symbols (symbol package)
-          (when (and (eq (symbol-package symbol) package)
-                     (or (not external-only)
-                         (eq (nth-value 1 (find-symbol (symbol-name symbol)
-                                                       package))
-                             :external))
+          (when (and (or (not external-only)
+                         (and (eq (symbol-package symbol) package)
+                              (eq (nth-value 1 (find-symbol (symbol-name symbol)
+                                                            package))
+                                  :external)))
                      (search string (symbol-name symbol) :test #'char-equal))
-            (push symbol result)))
+            (pushnew symbol result)))
         (sort result #'string-lessp))
-      (mapcan (lambda (package)
-                (apropos-list string-designator package external-only))
-              (sort (list-all-packages) #'string-lessp :key #'package-name))))
+      (delete-duplicates
+       (mapcan (lambda (package)
+                 (apropos-list string-designator package external-only))
+               (sort (list-all-packages) #'string-lessp :key #'package-name)))))
 
 (defun apropos (string-designator &optional package external-only)
   #!+sb-doc
@@ -1615,7 +1604,9 @@ PACKAGE."
               (package-%locally-nicknamed-by pkg) nil
               (package-source-location pkg) nil
               (gethash (package-%name pkg) names) pkg)
-        (%enter-new-nicknames pkg (package-%nicknames pkg))
+        (let ((nicks (package-%nicknames pkg)))
+          (setf (package-%nicknames pkg) nil) ; each is pushed in again
+          (%enter-new-nicknames pkg nicks))
         #!+sb-package-locks
         (setf (package-lock pkg) nil
               (package-%implementation-packages pkg) nil))))
