@@ -107,7 +107,7 @@
                 "SRC;PCL;MACROS"
                 "SRC;PCL;COMPILER-SUPPORT"
                 "SRC;PCL;LOW"
-                "SRC;PCL;SLOT-NAME"
+                #+nil "SRC;PCL;SLOT-NAME" ; moved to build-order.lisp-expr
                 "SRC;PCL;DEFCLASS"
                 "SRC;PCL;DEFS"
                 "SRC;PCL;FNGEN"
@@ -208,11 +208,43 @@
                             (load output-truename))
                     (error "LOAD of ~S failed." output-truename))
                   (sb-int:/show "done loading" output-truename))))))))
-  (with-compilation-unit ()
-    (do-srcs pcl-srcs))
-  (when *compile-files-p*
-    (format t "~&; Done with PCL compilation~2%"))
-  (do-srcs other-srcs)))
+
+   (let* ((ppd (copy-pprint-dispatch))
+          (sb-debug:*debug-print-variable-alist*
+           (list (cons '*print-pprint-dispatch* ppd)))
+          (*print-pprint-dispatch* ppd))
+     (set-pprint-dispatch
+      'sb-kernel:layout (lambda (stream obj)
+                          (print-unreadable-object (obj stream :type t)
+                            (write (sb-int:awhen (sb-kernel:layout-classoid obj)
+                                     (sb-kernel:classoid-name sb-int:it))
+                                   :stream stream))))
+     (set-pprint-dispatch
+      'sb-kernel:classoid (lambda (stream obj)
+                            (print-unreadable-object (obj stream :type t)
+                              (write (sb-kernel:classoid-name obj) :stream stream))))
+     (set-pprint-dispatch
+      'package (lambda (stream obj)
+                 (print-unreadable-object (obj stream :type t)
+                   (write (package-name obj) :stream stream))))
+     (set-pprint-dispatch
+      'pathname (lambda (stream obj)
+                  (write-string "#P" stream)
+                  (write (namestring obj) :stream stream)))
+     (set-pprint-dispatch
+      'sb-thread:thread (lambda (stream obj)
+                          (declare (ignore obj))
+                          (write-string "#<main-thread>" stream)))
+     (set-pprint-dispatch
+      'restart (lambda (stream obj)
+                 (print-unreadable-object (obj stream :type t :identity t)
+                   (write (restart-name obj) :stream stream))))
+     (with-compilation-unit ()
+       (let ((*compile-print* nil))
+         (do-srcs pcl-srcs)))
+     (when *compile-files-p*
+       (format t "~&; Done with PCL compilation~2%"))
+     (do-srcs other-srcs))))
 
 ;;;; setting package documentation
 
