@@ -160,20 +160,30 @@
   *universal-type*)
 
 (deftransform array-in-bounds-p ((array &rest subscripts))
-  (flet ((give-up ()
-           (give-up-ir1-transform
-            "~@<lower array bounds unknown or negative and upper bounds not ~
-             negative~:@>"))
-         (bound-known-p (x)
-           (integerp x))) ; might be NIL or *
-    (block nil
-      (let ((dimensions (array-type-dimensions-or-give-up
-                         (lvar-conservative-type array))))
+  (block nil
+    (flet ((give-up (&optional reason)
+             (cond ((= (length subscripts) 1)
+                    (let ((arg (sb!xc:gensym)))
+                      `(lambda (array ,arg)
+                         (and (typep ,arg '(and fixnum unsigned-byte))
+                              (< ,arg (array-dimension array 0))))))
+                   (t
+                    (give-up-ir1-transform
+                     (or reason
+                         "~@<lower array bounds unknown or negative and upper bounds not ~
+                         negative~:@>")))))
+           (bound-known-p (x)
+             (integerp x)))             ; might be NIL or *
+      (let ((dimensions (catch-give-up-ir1-transform
+                            ((array-type-dimensions-or-give-up
+                              (lvar-conservative-type array))
+                             args)
+                          (give-up (car args)))))
         ;; Might be *. (Note: currently this is never true, because the type
         ;; derivation infers the rank from the call to ARRAY-IN-BOUNDS-P, but
         ;; let's keep this future proof.)
         (when (eq '* dimensions)
-          (give-up-ir1-transform "array bounds unknown"))
+          (give-up "array bounds unknown"))
         ;; shortcut for zero dimensions
         (when (some (lambda (dim)
                       (and (bound-known-p dim) (zerop dim)))
@@ -198,7 +208,7 @@
                       ;; does not give us a definite clue either.
                       (give-up))
                      ((and (bound-known-p high) (minusp high))
-                      (return nil))     ; definitely below lower bound (zero).
+                      (return nil)) ; definitely below lower bound (zero).
                      (t
                       (cons low high))))))
           (let* ((subscripts-bounds (mapcar #'subscript-bounds subscripts))
@@ -226,7 +236,7 @@
                 t
                 (give-up))))))))
 
-(defoptimizer (aref derive-type) ((array &rest indices) node)
+(defoptimizer (aref derive-type) ((array &rest indices))
   (assert-array-rank array (length indices))
   (derive-aref-type array))
 
@@ -236,6 +246,7 @@
 
 (macrolet ((define (name)
              `(defoptimizer (,name derive-type) ((array index))
+                (declare (ignore index))
                 (derive-aref-type array))))
   (define hairy-data-vector-ref)
   (define hairy-data-vector-ref/check-bounds)
@@ -243,10 +254,12 @@
 
 #!+(or x86 x86-64)
 (defoptimizer (data-vector-ref-with-offset derive-type) ((array index offset))
+  (declare (ignore index offset))
   (derive-aref-type array))
 
 (macrolet ((define (name)
              `(defoptimizer (,name derive-type) ((array index new-value))
+                (declare (ignore index))
                 (assert-new-value-type new-value array))))
   (define hairy-data-vector-set)
   (define hairy-data-vector-set/check-bounds)
@@ -254,6 +267,7 @@
 
 #!+(or x86 x86-64)
 (defoptimizer (data-vector-set-with-offset derive-type) ((array index offset new-value))
+  (declare (ignore index offset))
   (assert-new-value-type new-value array))
 
 ;;; Figure out the type of the data vector if we know the argument
@@ -266,8 +280,10 @@
                         (array-type-specialized-element-type atype))
                       (*))))))
 (defoptimizer (%with-array-data derive-type) ((array start end))
+  (declare (ignore start end))
   (derive-%with-array-data/mumble-type array))
 (defoptimizer (%with-array-data/fp derive-type) ((array start end))
+  (declare (ignore start end))
   (derive-%with-array-data/mumble-type array))
 
 (defoptimizer (array-row-major-index derive-type) ((array &rest indices))
@@ -275,32 +291,35 @@
   *universal-type*)
 
 (defoptimizer (row-major-aref derive-type) ((array index))
+  (declare (ignore index))
   (derive-aref-type array))
 
 (defoptimizer (%set-row-major-aref derive-type) ((array index new-value))
+  (declare (ignore index))
   (assert-new-value-type new-value array))
 
-(defoptimizer (make-array derive-type)
-              ((dims &key initial-element element-type initial-contents
-                adjustable fill-pointer displaced-index-offset displaced-to))
+(defun derive-make-array-type (dims element-type adjustable
+                               fill-pointer displaced-to)
   (let* ((simple (and (unsupplied-or-nil adjustable)
                       (unsupplied-or-nil displaced-to)
                       (unsupplied-or-nil fill-pointer)))
          (spec
-          (or `(,(if simple 'simple-array 'array)
+           (or `(,(if simple 'simple-array 'array)
                  ,(cond ((not element-type) t)
+                        ((ctype-p element-type)
+                         (type-specifier element-type))
                         ((constant-lvar-p element-type)
                          (let ((ctype (careful-specifier-type
                                        (lvar-value element-type))))
                            (cond
-                             ((or (null ctype) (unknown-type-p ctype)) '*)
+                             ((or (null ctype) (contains-unknown-type-p ctype)) '*)
                              (t (sb!xc:upgraded-array-element-type
                                  (lvar-value element-type))))))
                         (t
                          '*))
                  ,(cond ((constant-lvar-p dims)
                          (let* ((val (lvar-value dims))
-                                (cdims (if (listp val) val (list val))))
+                                (cdims (ensure-list val)))
                            (if simple
                                cdims
                                (length cdims))))
@@ -309,13 +328,31 @@
                          '(*))
                         (t
                          '*)))
-              'array)))
+               'array)))
     (if (and (not simple)
              (or (supplied-and-true adjustable)
                  (supplied-and-true displaced-to)
                  (supplied-and-true fill-pointer)))
         (careful-specifier-type `(and ,spec (not simple-array)))
         (careful-specifier-type spec))))
+
+(defoptimizer (make-array derive-type)
+    ((dims &key element-type adjustable fill-pointer displaced-to))
+  (derive-make-array-type dims element-type adjustable
+                          fill-pointer displaced-to))
+
+(defoptimizer (%make-array derive-type)
+    ((dims widetag n-bits &key adjustable fill-pointer displaced-to))
+  (declare (ignore n-bits))
+  (let ((saetp (and (constant-lvar-p widetag)
+                    (find (lvar-value widetag)
+                          sb!vm:*specialized-array-element-type-properties*
+                          :key #'sb!vm:saetp-typecode))))
+    (derive-make-array-type dims (if saetp
+                                     (sb!vm:saetp-ctype saetp)
+                                     *wild-type*)
+                            adjustable fill-pointer displaced-to)))
+
 
 ;;;; constructors
 
@@ -333,19 +370,37 @@
                        ,@(when initial-element
                            '(:initial-element initial-element)))))
 
+;; Traverse the :INTIAL-CONTENTS argument to an array constructor call,
+;; changing the skeleton of the data to be constructed by calls to LIST
+;; and wrapping some declarations around each array cell's constructor.
+;; If a macro is involved, expand it before traversing.
+;; Known bugs:
+;; - Despite the effort to handle multidimensional arrays here,
+;;   an array-header will not be stack-allocated, so the data won't be either.
+;; - inline functions whose behavior is merely to call LIST don't work
+;;   e.g. :INITIAL-CONTENTS (MY-LIST a b) ; where MY-LIST is inline
+;;                                        ; and effectively just (LIST ...)
 (defun rewrite-initial-contents (rank initial-contents env)
-  (if (plusp rank)
-      (if (and (consp initial-contents)
-               (member (car initial-contents) '(list vector sb!impl::backq-list)))
-          `(list ,@(mapcar (lambda (dim)
-                             (rewrite-initial-contents (1- rank) dim env))
-                           (cdr initial-contents)))
-          initial-contents)
+  (named-let recurse ((rank rank) (data initial-contents))
+    (declare (type index rank))
+    (if (plusp rank)
+        (flet ((sequence-constructor-p (form)
+                 (member (car form) '(sb!impl::|List| list
+                                      sb!impl::|Vector| vector))))
+          (let (expanded)
+            (cond ((not (listp data)) data)
+                  ((sequence-constructor-p data)
+                   `(list ,@(mapcar (lambda (dim) (recurse (1- rank) dim))
+                                    (cdr data))))
+                  ((and (sb!xc:macro-function (car data) env)
+                        (listp (setq expanded (sb!xc:macroexpand data env)))
+                        (sequence-constructor-p expanded))
+                   (recurse rank expanded))
+                  (t data))))
       ;; This is the important bit: once we are past the level of
       ;; :INITIAL-CONTENTS that relates to the array structure, reinline LIST
       ;; and VECTOR so that nested DX isn't screwed up.
-      `(locally (declare (inline list vector))
-         ,initial-contents)))
+        `(locally (declare (inline list vector)) ,data))))
 
 ;;; Prevent open coding DIMENSION and :INITIAL-CONTENTS arguments, so that we
 ;;; can pick them apart in the DEFTRANSFORMS, and transform '(3) style
@@ -358,7 +413,7 @@
       (multiple-value-bind (new-dimensions rank)
           (flet ((constant-dims (dimensions)
                    (let* ((dims (constant-form-value dimensions env))
-                          (canon (if (listp dims) dims (list dims)))
+                          (canon (ensure-list dims))
                           (rank (length canon)))
                      (values (if (= rank 1)
                                  (list 'quote (car canon))
@@ -378,6 +433,31 @@
         `(locally (declare (notinline list vector))
            (make-array ,new-dimensions ,@keyargs)))))
 
+(define-source-transform coerce (x type &environment env)
+  (if (and (sb!xc:constantp type env)
+           (proper-list-p x)
+           (memq (car x) '(sb!impl::|List| list
+                           sb!impl::|Vector| vector)))
+      (let* ((type (constant-form-value type env))
+             (length (1- (length x)))
+             ;; Special case, since strings are unions
+             (string-p (member type '(string simple-string)))
+             (ctype (or string-p
+                        (careful-values-specifier-type type))))
+        (if (or string-p
+                (and (array-type-p ctype)
+                     (csubtypep ctype (specifier-type '(array * (*))))
+                     (proper-list-of-length-p (array-type-dimensions ctype) 1)
+                     (or (eq (car (array-type-dimensions ctype)) '*)
+                         (eq (car (array-type-dimensions ctype)) length))))
+            `(make-array ,length
+                         :element-type ',(if string-p
+                                             'character
+                                             (nth-value 1 (simplify-vector-type ctype)))
+                         :initial-contents ,x)
+            (values nil t)))
+      (values nil t)))
+
 ;;; This baby is a bit of a monster, but it takes care of any MAKE-ARRAY
 ;;; call which creates a vector with a known element type -- and tries
 ;;; to do a good job with all the different ways it can happen.
@@ -414,12 +494,13 @@
                   (t
                    (let ((n-elements-per-word (/ sb!vm:n-word-bits n-bits)))
                      (declare (type index n-elements-per-word)) ; i.e., not RATIO
-                     `(ceiling ,padded-length-form ,n-elements-per-word)))))))
+                     `(ceiling (truly-the index ,padded-length-form)
+                               ,n-elements-per-word)))))))
          (result-spec
           `(simple-array ,(sb!vm:saetp-specifier saetp) (,(or c-length '*))))
          (alloc-form
-          `(truly-the ,result-spec
-                      (allocate-vector ,typecode (the index length) ,n-words-form))))
+           `(truly-the ,result-spec
+                       (allocate-vector ,typecode (the index length) ,n-words-form))))
     (cond ((and initial-element initial-contents)
            (abort-ir1-transform "Both ~S and ~S specified."
                                 :initial-contents :initial-element))
@@ -427,7 +508,8 @@
           ;; constant LENGTH.
           ((and initial-contents c-length
                 (lvar-matches initial-contents
-                              :fun-names '(list vector sb!impl::backq-list)
+                              :fun-names '(list vector
+                                           sb!impl::|List| sb!impl::|Vector|)
                               :arg-count c-length))
            (let ((parameters (eliminate-keyword-args
                               call 1 '((:element-type element-type)
@@ -523,9 +605,9 @@
 
 (deftransform make-array ((dims &key initial-element element-type
                                      adjustable fill-pointer)
-                          (t &rest *))
-  (when (null initial-element)
-    (give-up-ir1-transform))
+                          (t &rest *) *
+                          :node node)
+  (delay-ir1-transform node :constraint)
   (let* ((eltype (cond ((not element-type) t)
                        ((not (constant-lvar-p element-type))
                         (give-up-ir1-transform
@@ -533,22 +615,28 @@
                        (t
                         (lvar-value element-type))))
          (eltype-type (ir1-transform-specifier-type eltype))
-         (saetp (find-if (lambda (saetp)
-                           (csubtypep eltype-type (sb!vm:saetp-ctype saetp)))
-                         sb!vm:*specialized-array-element-type-properties*))
-         (creation-form `(make-array dims
-                          :element-type ',(type-specifier (sb!vm:saetp-ctype saetp))
+         (saetp (if (unknown-type-p eltype-type)
+                    (give-up-ir1-transform
+                     "ELEMENT-TYPE ~s is not a known type"
+                     eltype-type)
+                    (find eltype-type
+                          sb!vm:*specialized-array-element-type-properties*
+                          :key #'sb!vm:saetp-ctype
+                          :test #'csubtypep)))
+         (creation-form `(%make-array
+                          dims
+                          ,(if saetp
+                               (sb!vm:saetp-typecode saetp)
+                               (give-up-ir1-transform))
+                          ,(sb!vm:saetp-n-bits saetp)
                           ,@(when fill-pointer
-                                  '(:fill-pointer fill-pointer))
+                              '(:fill-pointer fill-pointer))
                           ,@(when adjustable
-                                  '(:adjustable adjustable)))))
-
-    (unless saetp
-      (give-up-ir1-transform "ELEMENT-TYPE not found in *SAETP*: ~S" eltype))
-
-    (cond ((and (constant-lvar-p initial-element)
-                (eql (lvar-value initial-element)
-                     (sb!vm:saetp-initial-element-default saetp)))
+                              '(:adjustable adjustable)))))
+    (cond ((or (not initial-element)
+               (and (constant-lvar-p initial-element)
+                    (eql (lvar-value initial-element)
+                         (sb!vm:saetp-initial-element-default saetp))))
            creation-form)
           (t
            ;; error checking for target, disabled on the host because
@@ -575,10 +663,10 @@
                   (compiler-style-warn "~S is not a ~S."
                                        value eltype)))))
            `(let ((array ,creation-form))
-             (multiple-value-bind (vector)
-                 (%data-vector-and-index array 0)
-               (fill vector (the ,(sb!vm:saetp-specifier saetp) initial-element)))
-             array)))))
+              (multiple-value-bind (vector)
+                  (%data-vector-and-index array 0)
+                (fill vector (the ,(sb!vm:saetp-specifier saetp) initial-element)))
+              array)))))
 
 ;;; The list type restriction does not ensure that the result will be a
 ;;; multi-dimensional array. But the lack of adjustable, fill-pointer,
@@ -613,9 +701,9 @@
           (element-type-ctype (and (constant-lvar-p element-type)
                                    (ir1-transform-specifier-type
                                     (lvar-value element-type)))))
-      (when (unknown-type-p element-type-ctype)
+      (when (contains-unknown-type-p element-type-ctype)
         (give-up-ir1-transform))
-      (unless (every #'integerp dims)
+      (unless (every (lambda (x) (typep x '(integer 0))) dims)
         (give-up-ir1-transform
          "The dimension list contains something other than an integer: ~S"
          dims))
@@ -721,20 +809,60 @@
     ;; FIXME: intersection type
     (t :maybe)))
 
-;;; If we can tell the rank from the type info, use it instead.
+;; Let type derivation handle constant cases. We only do easy strength
+;; reduction.
 (deftransform array-rank ((array) (array) * :node node)
   (let ((array-type (lvar-type array)))
-    (let ((dims (array-type-dimensions-or-give-up array-type)))
-      (cond ((listp dims)
-             (length dims))
-            ((eq t (and (array-type-p array-type)
-                        (array-type-complexp array-type)))
-             '(%array-rank array))
-            (t
-             (delay-ir1-transform node :constraint)
-             `(if (array-header-p array)
-                  (%array-rank array)
-                  1))))))
+    (cond ((eq t (and (array-type-p array-type)
+                      (array-type-complexp array-type)))
+           '(%array-rank array))
+          (t
+           (delay-ir1-transform node :constraint)
+           `(if (array-header-p array)
+                (%array-rank array)
+                1)))))
+
+(defun derive-array-rank (ctype)
+  (let ((array (specifier-type 'array)))
+    (flet ((over (x)
+             (cond ((not (types-equal-or-intersect x array))
+                    '()) ; Definitely not an array!
+                   ((array-type-p x)
+                    (let ((dims (array-type-dimensions x)))
+                      (if (eql dims '*)
+                          '*
+                          (list (length dims)))))
+                   (t '*)))
+           (under (x)
+             ;; Might as well catch some easy negation cases.
+             (typecase x
+               (array-type
+                (let ((dims (array-type-dimensions x)))
+                  (cond ((eql dims '*)
+                         '*)
+                        ((every (lambda (dim)
+                                  (eql dim '*))
+                                dims)
+                         (list (length dims)))
+                        (t
+                         '()))))
+               (t '()))))
+      (declare (dynamic-extent #'over #'under))
+      (multiple-value-bind (not-p ranks)
+          (list-abstract-type-function ctype #'over :under #'under)
+        (cond ((eql ranks '*)
+               (aver (not not-p))
+               nil)
+              (not-p
+               (specifier-type `(not (member ,@ranks))))
+              (t
+               (specifier-type `(member ,@ranks))))))))
+
+(defoptimizer (array-rank derive-type) ((array))
+  (derive-array-rank (lvar-type array)))
+
+(defoptimizer (%array-rank derive-type) ((array))
+  (derive-array-rank (lvar-type array)))
 
 ;;; If we know the dimensions at compile time, just use it. Otherwise,
 ;;; if we can tell that the axis is in bounds, convert to
@@ -871,12 +999,12 @@
 ;;; do type testing inside %WITH-ARRAY-DATA instead of outside, and
 ;;; the DEFTRANSFORM can't tell that that's going on, so it can make
 ;;; sense to use FORCE-INLINE option in that case.
-(def!macro with-array-data (((data-var array &key offset-var)
-                             (start-var &optional (svalue 0))
-                             (end-var &optional (evalue nil))
-                             &key force-inline check-fill-pointer)
-                            &body forms
-                            &environment env)
+(sb!xc:defmacro with-array-data (((data-var array &key offset-var)
+                                  (start-var &optional (svalue 0))
+                                  (end-var &optional (evalue nil))
+                                  &key force-inline check-fill-pointer)
+                                 &body forms
+                                 &environment env)
   (once-only ((n-array array)
               (n-svalue `(the index ,svalue))
               (n-evalue `(the (or index null) ,evalue)))
@@ -938,7 +1066,7 @@
                     (the index ,cumulative-offset)))
          (declare (type index ,cumulative-offset))))))
 
-(defun transform-%with-array-data/muble (array node check-fill-pointer)
+(defun transform-%with-array-data/mumble (array node check-fill-pointer)
   (let ((element-type (upgraded-element-type-specifier-or-give-up array))
         (type (lvar-type array))
         (check-bounds (policy node (plusp insert-array-bounds-checks))))
@@ -978,14 +1106,14 @@
                                 :node node
                                 :policy (> speed space))
   "inline non-SIMPLE-vector-handling logic"
-  (transform-%with-array-data/muble array node nil))
+  (transform-%with-array-data/mumble array node nil))
 (deftransform %with-array-data/fp ((array start end)
                                 ((or vector simple-array) index (or index null) t)
                                 *
                                 :node node
                                 :policy (> speed space))
   "inline non-SIMPLE-vector-handling logic"
-  (transform-%with-array-data/muble array node t))
+  (transform-%with-array-data/mumble array node t))
 
 ;;;; array accessors
 
@@ -1119,14 +1247,16 @@
   (let* ((type (lvar-type array))
          (element-ctype (array-type-upgraded-element-type type)))
     (cond
+      ((eq element-ctype *empty-type*)
+       `(data-nil-vector-ref array index))
       ((and (array-type-p type)
             (null (array-type-complexp type))
-            (not (eql element-ctype *wild-type*))
+            (neq element-ctype *wild-type*)
             (eql (length (array-type-dimensions type)) 1))
        (let* ((declared-element-ctype (array-type-declared-element-type type))
               (bare-form
-               `(data-vector-ref array
-                 (%check-bound array (array-dimension array 0) index))))
+                `(data-vector-ref array
+                                  (%check-bound array (array-dimension array 0) index))))
          (if (type= declared-element-ctype element-ctype)
              bare-form
              `(the ,(type-specifier declared-element-ctype) ,bare-form))))
@@ -1153,7 +1283,7 @@
                   ;; use that information it for type checking (even
                   ;; if the access can't be optimized due to the array
                   ;; not being simple).
-                  (when (and (eql element-type *wild-type*)
+                  (when (and (eq element-type *wild-type*)
                              ;; This type logic corresponds to the special
                              ;; case for strings in HAIRY-DATA-VECTOR-REF
                              ;; (generic/vm-tran.lisp)
