@@ -13,12 +13,18 @@
 
 (/show0 "compiler-deftype.lisp 14")
 
-(defun %compiler-deftype (name lambda-list expander doc source-location)
+(defun %compiler-deftype (name lambda-list expander source-location &optional doc)
+  #+sb-xc-host (declare (ignore lambda-list))
   (with-single-package-locked-error
       (:symbol name "defining ~A as a type specifier"))
   (ecase (info :type :kind name)
     (:primitive
-     (when *type-system-initialized*
+     ;; The allowance for already-processed deftypes works around the issue that
+     ;; building the cross-compiler made these :primitive and not re-definable.
+     ;; One remedy was to claim that they weren't :primitive, which made the xc
+     ;; model of the target's type system not a faithful reflection.
+     (when (and *type-system-initialized*
+                #+sb-xc-host (not (member name !*xc-processed-deftypes*)))
        (error "illegal to redefine standard type: ~S" name)))
     (:instance
      (warn "The class ~S is being redefined to be a DEFTYPE." name)
@@ -41,8 +47,8 @@
      )
     ((nil :forthcoming-defclass-type)
      (setf (info :type :kind name) :defined)))
-  (setf (info :type :expander name) expander
-        (info :type :lambda-list name) lambda-list)
+  #-sb-xc-host (setf (%fun-lambda-list expander) lambda-list)
+  (setf (info :type :expander name) expander)
   (sb!c:with-source-location (source-location)
     (setf (info :type :source-location name) source-location))
   (when doc
