@@ -344,7 +344,7 @@
   (let* ((vop (tn-ref-vop op))
          (args (vop-args vop))
          (results (vop-results vop))
-         (name (with-output-to-string (stream)
+         (name (with-simple-output-to-string (stream)
                  (print-tn-guts tn stream)))
          (2comp (component-info *component-being-compiled*))
          temp)
@@ -1322,7 +1322,7 @@
                           &rest keys &key limit reads writes)
                          &body body)
   (declare (ignore limit reads writes))
-  (let ((callback (gensym "CALLBACK")))
+  (let ((callback (sb!xc:gensym "CALLBACK")))
     `(flet ((,callback (,target-variable)
               ,@body))
        (declare (dynamic-extent #',callback))
@@ -1542,20 +1542,9 @@
       (let ((size (sb-size sb)))
         (fill (finite-sb-always-live sb) nil)
         (setf (finite-sb-always-live sb)
-              (make-array size
-                          :initial-element
-                          #-sb-xc #*
-                          ;; The cross-compiler isn't very good at
-                          ;; dumping specialized arrays, so we delay
-                          ;; construction of this SIMPLE-BIT-VECTOR
-                          ;; until runtime.
-                          #+sb-xc (make-array 0 :element-type 'bit)))
+              (make-array size :initial-element #*))
         (setf (finite-sb-always-live-count sb)
-              (make-array size
-                          :initial-element
-                          #-sb-xc #*
-                          ;; Ibid
-                          #+sb-xc (make-array 0 :element-type 'fixnum)))
+              (make-array size :initial-element 0))
 
         (fill (finite-sb-conflicts sb) nil)
         (setf (finite-sb-conflicts sb)
@@ -1626,14 +1615,6 @@
                  (setf speed-3 t)
                  (return)))))
 
-         ;; Call the target functions.
-         (do-ir2-blocks (block component)
-           (do ((vop (ir2-block-start-vop block) (vop-next vop)))
-               ((null vop))
-             (let ((target-fun (vop-info-target-fun (vop-info vop))))
-               (when target-fun
-                 (funcall target-fun vop)))))
-
          ;; Assign costs to normal TNs so we know which ones should always
          ;; be packed on the stack, and which are important not to spill.
          (when *pack-assign-costs*
