@@ -60,7 +60,8 @@
 (deftransform typep ((object type &optional env) * * :node node)
   (unless (constant-lvar-p type)
     (give-up-ir1-transform "can't open-code test of non-constant type"))
-  (unless (and (constant-lvar-p env) (null (lvar-value env)))
+  (unless (or (null env)
+              (and (constant-lvar-p env) (null (lvar-value env))))
     (give-up-ir1-transform "environment argument present and not null"))
   (multiple-value-bind (expansion fail-p)
       (source-transform-typep 'object (lvar-value type))
@@ -357,19 +358,52 @@
 ;;; If necessary recurse to check the cons type.
 (defun source-transform-cons-typep (object type)
   (let* ((car-type (cons-type-car-type type))
-         (cdr-type (cons-type-cdr-type type)))
-    (let ((car-test-p (not (type= car-type *universal-type*)))
-          (cdr-test-p (not (type= cdr-type *universal-type*))))
-      (if (and (not car-test-p) (not cdr-test-p))
-          `(consp ,object)
-          (once-only ((n-obj object))
-            `(and (consp ,n-obj)
-                  ,@(if car-test-p
-                        `((typep (car ,n-obj)
-                                 ',(type-specifier car-type))))
-                  ,@(if cdr-test-p
-                        `((typep (cdr ,n-obj)
-                                 ',(type-specifier cdr-type))))))))))
+         (cdr-type (cons-type-cdr-type type))
+         (car-test-p (not (type= car-type *universal-type*)))
+         (cdr-test-p (not (type= cdr-type *universal-type*))))
+    (if (and (not car-test-p) (not cdr-test-p))
+        `(consp ,object)
+        ;; CONSP can be safely weakened to LISTP if either of the CAR
+        ;; or CDR test (or both) can distinguish LIST from CONS
+        ;; by never returning T when given an input of NIL.
+        (labels ((safely-weakened (ctype)
+                   (typecase ctype
+                     (member-type
+                      (not (member nil (member-type-members ctype))))
+                     (classoid
+                      ;; can't weaken if the specifier is (CONS SYMBOL)
+                      (not (ctypep nil ctype)))
+                     ;; these are disjoint from NIL
+                     ((or cons-type numeric-type array-type character-set-type)
+                      t)
+                     (intersection-type
+                      ;; at least one of them must not spuriously return T
+                      (some #'safely-weakened (compound-type-types ctype)))
+                     (union-type
+                      ;; require that none spuriously return T
+                      (every #'safely-weakened (compound-type-types ctype)))
+                     (hairy-type
+                      ;; hack - (CONS KEYWORD) is weakenable
+                      ;; because NIL is not a keyword.
+                      (equal (hairy-type-specifier ctype)
+                             '(satisfies keywordp))))))
+          (let* ((n-obj (sb!xc:gensym))
+                 (car-test
+                  (and car-test-p
+                       `((typep (car ,n-obj) ',(type-specifier car-type)))))
+                 (cdr-test
+                  (and cdr-test-p
+                       `((typep (cdr ,n-obj) ',(type-specifier cdr-type))))))
+            `(let ((,n-obj ,object))
+               ;; Being paranoid, perform the safely weakenable test first
+               ;; so that the other part doesn't execute on an object that
+               ;; it would not have gotten, were the CONSP test not weakened.
+               ,(cond ((and car-test-p (safely-weakened car-type))
+                       `(and (listp ,n-obj) ,@car-test ,@cdr-test))
+                      ((and cdr-test-p (safely-weakened cdr-type))
+                       `(and (listp ,n-obj) ,@cdr-test ,@car-test))
+                      (t
+                       `(and (consp ,n-obj) ,@car-test ,@cdr-test)))))))))
 
 (defun source-transform-character-set-typep (object type)
   (let ((pairs (character-set-type-pairs type)))
@@ -551,6 +585,13 @@
        ;; Delay the type transform to give type propagation a chance.
        (delay-ir1-transform node :constraint)
 
+       ;; FIXME: (TYPEP X 'ERROR) - or any condition - checks whether X
+       ;; has the lowtag of either an ordinary or funcallable instance.
+       ;; But you can not define a class that is both CONDITION and FUNCTION
+       ;; because CONDITION-CLASS and FUNCALLABLE-STANDARD-CLASS are
+       ;; incompatible metaclasses. Thus the type test is less efficient than
+       ;; could be, since fun-pointer-lowtag can not occur in the "true" case.
+
        ;; Otherwise transform the type test.
        (binding* (((pred get-layout)
                    (cond ((csubtypep class (specifier-type 'funcallable-instance))
@@ -602,20 +643,18 @@
                     (let ((dd (layout-info layout)))
                       (and dd
                            (not (dd-default-constructor dd))
-                           (let ((ctors (sb!kernel::dd-constructors dd)))
+                           (let ((ctors (dd-constructors dd)))
                              (or (not ctors) (equal ctors '((nil))))))))
-                   (hierarchy-check
+                   (get-ancestor
                     ;; Use DATA-VECTOR-REF directly, since that's what SVREF in
                     ;; a SAFETY 0 lexenv will eventually be transformed to.
                     ;; This can give a large compilation speedup, since
                     ;; %INSTANCE-TYPEPs are frequently created during
                     ;; GENERATE-TYPE-CHECKS, and the normal aref transformation
                     ;; path is pretty heavy.
-                    `(and (> (layout-depthoid ,n-layout) ,depthoid)
-                          (locally (declare (optimize (safety 0)))
-                            (eq (data-vector-ref (layout-inherits ,n-layout)
-                                                 ,depthoid)
-                                ',layout)))))
+                    `(locally (declare (optimize (safety 0)))
+                       (data-vector-ref (layout-inherits ,n-layout) ,depthoid)))
+                   (deeper-p `(> (layout-depthoid ,n-layout) ,depthoid)))
               (aver (equal pred '(%instancep object)))
               `(and ,pred
                     (let ((,n-layout ,get-layout))
@@ -627,9 +666,9 @@
                       ;; object with an invalid layout to a structure
                       ;; type test.
                       ,(if abstract-base-p
-                           `(if ,hierarchy-check t (eq ,n-layout ',layout))
-                           `(if (eq ,n-layout ',layout) t ,hierarchy-check))))))
-
+                           `(eq (if ,deeper-p ,get-ancestor ,n-layout) ,layout)
+                           `(cond ((eq ,n-layout ,layout) t)
+                                  (,deeper-p (eq ,get-ancestor ,layout))))))))
            ((and layout (>= (layout-depthoid layout) 0))
             ;; hierarchical layout depths for other things (e.g.
             ;; CONDITION, STREAM)
@@ -645,15 +684,13 @@
                     `((when (layout-invalid ,n-layout)
                         (setq ,n-layout (update-object-layout-or-invalid
                                          object ',layout)))
-                      (if (let ((,n-inherits (layout-inherits
-                                              (truly-the layout ,n-layout))))
-                            (declare (optimize (safety 0)))
-                            (and (> (vector-length ,n-inherits) ,depthoid)
-                                 ;; See above.
-                                 (eq (data-vector-ref ,n-inherits ,depthoid)
-                                     ',layout)))
-                          t
-                          (eq ,n-layout ',layout)))))
+                      (let ((,n-inherits (layout-inherits
+                                          (truly-the layout ,n-layout))))
+                        (declare (optimize (safety 0)))
+                        (eq (if (> (vector-length ,n-inherits) ,depthoid)
+                                (data-vector-ref ,n-inherits ,depthoid)
+                                ,n-layout)
+                            ,layout)))))
               (if pred
                   `(and ,pred (let ((,n-layout ,get-layout)) ,@guts))
                   `(block typep
@@ -863,7 +900,11 @@
            ((csubtypep tspec (specifier-type 'function))
             (if (csubtypep (lvar-type x) (specifier-type 'symbol))
                 `(coerce-symbol-to-fun x)
-                `(coerce-to-fun x)))
+                ;; if X can later be derived as FUNCTION then we don't want
+                ;; to call COERCE-TO-FUN, because there's no smartness
+                ;; that can undo that and see that it's really (IDENTITY X).
+                (progn (delay-ir1-transform node :constraint)
+                       `(coerce-to-fun x))))
            (t
             (give-up-ir1-transform
              "~@<open coding coercion to ~S not implemented.~:@>"
