@@ -309,7 +309,7 @@
   (:result-types positive-fixnum)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
-    ;; cdr slot, so we have to strip off the three low bits to make sure
+    ;; cdr slot, so we have to zero the fixnum tag bit(s) to make sure
     ;; it is a fixnum.  The lowtag selection magic that is required to
     ;; ensure this is explained in the comment in objdef.lisp
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
@@ -378,16 +378,17 @@
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg) :target tmp
                  :to :load))
-  (:temporary (:sc unsigned-reg) tls-index bsp tmp)
+  (:temporary (:sc unsigned-reg :offset rax-offset) tls-index)
+  (:temporary (:sc unsigned-reg) bsp tmp)
   (:generator 10
     (load-binding-stack-pointer bsp)
     (inst mov (reg-in-size tls-index :dword) (tls-index-of symbol))
     (inst add bsp (* binding-size n-word-bytes))
     (store-binding-stack-pointer bsp)
-    (inst test tls-index tls-index)
+    (inst test (reg-in-size tls-index :dword) (reg-in-size tls-index :dword))
     (inst jmp :ne TLS-INDEX-VALID)
     (inst mov tls-index symbol)
-    (inst mov tmp (subprimitive-tls-allocator tls-index))
+    (inst mov tmp (make-fixup 'alloc-tls-index :assembly-routine))
     (inst call tmp)
     TLS-INDEX-VALID
     (inst mov tmp (make-ea :qword :base thread-base-tn :index tls-index))
@@ -489,9 +490,6 @@
     (loadw symbol bsp binding-symbol-slot)
     (inst test symbol symbol)
     (inst jmp :z SKIP)
-    ;; Bind stack debug sentinels have the unbound marker in the symbol slot
-    (inst cmp symbol unbound-marker-widetag)
-    (inst jmp :eq SKIP)
     (loadw value bsp binding-value-slot)
     #!-sb-thread
     (storew value symbol symbol-value-slot other-pointer-lowtag)
@@ -508,27 +506,6 @@
     (store-binding-stack-pointer bsp)
 
     DONE))
-
-(define-vop (bind-sentinel)
-  (:temporary (:sc unsigned-reg) bsp)
-  (:generator 1
-     (load-binding-stack-pointer bsp)
-     (inst add bsp (* binding-size n-word-bytes))
-     (storew unbound-marker-widetag bsp (- binding-symbol-slot binding-size))
-     (storew rbp-tn bsp (- binding-value-slot binding-size))
-     (store-binding-stack-pointer bsp)))
-
-(define-vop (unbind-sentinel)
-  (:temporary (:sc unsigned-reg) bsp)
-  (:generator 1
-     (load-binding-stack-pointer bsp)
-     (storew 0 bsp (- binding-value-slot binding-size))
-     (storew 0 bsp (- binding-symbol-slot binding-size))
-     (inst sub bsp (* binding-size n-word-bytes))
-     (store-binding-stack-pointer bsp)))
-
-
-
 
 ;;;; closure indexing
 
@@ -597,45 +574,31 @@
 
 ;;;; raw instance slot accessors
 
-(defun make-ea-for-raw-slot (object instance-length
-                             &key (index nil) (adjustment 0) (scale 1))
-  (if (integerp instance-length)
-      ;; For RAW-INSTANCE-INIT/* VOPs, which know the exact instance length
-      ;; at compile time.
-      (make-ea :qword
-               :base object
-               :disp (+ (* (- instance-length instance-slots-offset index)
-                           n-word-bytes)
-                        (- instance-pointer-lowtag)
-                        adjustment))
-      (etypecase index
-        (null
-         (make-ea :qword :base object :index instance-length :scale scale
-                  :disp (+ (* (1- instance-slots-offset) n-word-bytes)
-                           (- instance-pointer-lowtag)
-                           adjustment)))
-        (integer
-         (make-ea :qword :base object :index instance-length
-                  :scale 8
-                  :disp (+ (* (1- instance-slots-offset) n-word-bytes)
-                           (- instance-pointer-lowtag)
-                           adjustment
-                           (* index (- n-word-bytes))))))))
+(defun make-ea-for-raw-slot (object index)
+  (etypecase index
+    (integer
+       (make-ea :qword
+                :base object
+                :disp (+ (* (+ instance-slots-offset index)
+                            n-word-bytes)
+                         (- instance-pointer-lowtag))))
+    (tn
+       (make-ea :qword
+                :base object
+                :index index
+                :scale (ash 1 (- word-shift n-fixnum-tag-bits))
+                :disp (+ (* instance-slots-offset n-word-bytes)
+                         (- instance-pointer-lowtag))))))
 
 (define-vop (raw-instance-ref/word)
   (:translate %raw-instance-ref/word)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)) (index :scs (any-reg)))
   (:arg-types * tagged-num)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst mov value (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
+    (inst mov value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-ref-c/word)
   (:translate %raw-instance-ref/word)
@@ -645,13 +608,10 @@
                                              #.instance-pointer-lowtag
                                              #.instance-slots-offset)))
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst mov value (make-ea-for-raw-slot object tmp :index index))))
+    (inst mov value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-set/word)
   (:translate %raw-instance-set/word)
@@ -660,15 +620,10 @@
          (index :scs (any-reg))
          (value :scs (unsigned-reg) :target result))
   (:arg-types * tagged-num unsigned-num)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst mov (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))) value)
+    (inst mov (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-set-c/word)
@@ -681,22 +636,32 @@
                                              #.instance-slots-offset))
               unsigned-num)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst mov (make-ea-for-raw-slot object tmp :index index) value)
+    (inst mov (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-init/word)
   (:args (object :scs (descriptor-reg))
          (value :scs (unsigned-reg)))
   (:arg-types * unsigned-num)
-  (:info instance-length index)
+  (:info index)
   (:generator 4
-    (inst mov (make-ea-for-raw-slot object instance-length :index index) value)))
+    (inst mov (make-ea-for-raw-slot object index) value)))
+
+(define-vop (raw-instance-atomic-incf/word)
+  (:translate %raw-instance-atomic-incf/word)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg))
+         (index :scs (any-reg))
+         (diff :scs (unsigned-reg) :target result))
+  (:arg-types * tagged-num unsigned-num)
+  (:results (result :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:generator 5
+    (inst xadd (make-ea-for-raw-slot object index) diff :lock)
+    (move result diff)))
 
 (define-vop (raw-instance-atomic-incf-c/word)
   (:translate %raw-instance-atomic-incf/word)
@@ -708,13 +673,10 @@
                                              #.instance-slots-offset))
               unsigned-num)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst xadd (make-ea-for-raw-slot object tmp :index index) diff :lock)
+    (inst xadd (make-ea-for-raw-slot object index) diff :lock)
     (move result diff)))
 
 (define-vop (raw-instance-ref/single)
@@ -723,15 +685,10 @@
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (single-reg)))
   (:result-types single-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movss value (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
+    (inst movss value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-ref-c/single)
   (:translate %raw-instance-ref/single)
@@ -741,13 +698,10 @@
                                              #.instance-pointer-lowtag
                                              #.instance-slots-offset)))
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (single-reg)))
   (:result-types single-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movss value (make-ea-for-raw-slot object tmp :index index))))
+    (inst movss value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-set/single)
   (:translate %raw-instance-set/single)
@@ -756,15 +710,10 @@
          (index :scs (any-reg))
          (value :scs (single-reg) :target result))
   (:arg-types * positive-fixnum single-float)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (single-reg)))
   (:result-types single-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movss (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))) value)
+    (inst movss (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-set-c/single)
@@ -777,22 +726,19 @@
                                              #.instance-slots-offset))
               single-float)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (single-reg)))
   (:result-types single-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movss (make-ea-for-raw-slot object tmp :index index) value)
+    (inst movss (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-init/single)
   (:args (object :scs (descriptor-reg))
          (value :scs (single-reg)))
   (:arg-types * single-float)
-  (:info instance-length index)
+  (:info index)
   (:generator 4
-    (inst movss (make-ea-for-raw-slot object instance-length :index index) value)))
+    (inst movss (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-ref/double)
   (:translate %raw-instance-ref/double)
@@ -800,15 +746,10 @@
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (double-reg)))
   (:result-types double-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movsd value (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
+    (inst movsd value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-ref-c/double)
   (:translate %raw-instance-ref/double)
@@ -818,13 +759,10 @@
                                              #.instance-pointer-lowtag
                                              #.instance-slots-offset)))
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (double-reg)))
   (:result-types double-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movsd value (make-ea-for-raw-slot object tmp :index index))))
+    (inst movsd value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-set/double)
   (:translate %raw-instance-set/double)
@@ -833,15 +771,10 @@
          (index :scs (any-reg))
          (value :scs (double-reg) :target result))
   (:arg-types * positive-fixnum double-float)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (double-reg)))
   (:result-types double-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movsd (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))) value)
+    (inst movsd (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-set-c/double)
@@ -854,22 +787,19 @@
                                              #.instance-slots-offset))
               double-float)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (double-reg)))
   (:result-types double-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movsd (make-ea-for-raw-slot object tmp :index index) value)
+    (inst movsd (make-ea-for-raw-slot object index) value)
     (move result value)))
 
 (define-vop (raw-instance-init/double)
   (:args (object :scs (descriptor-reg))
          (value :scs (double-reg)))
   (:arg-types * double-float)
-  (:info instance-length index)
+  (:info index)
   (:generator 4
-    (inst movsd (make-ea-for-raw-slot object instance-length :index index) value)))
+    (inst movsd (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-ref/complex-single)
   (:translate %raw-instance-ref/complex-single)
@@ -877,15 +807,10 @@
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movq value (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
+    (inst movq value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-ref-c/complex-single)
   (:translate %raw-instance-ref/complex-single)
@@ -895,13 +820,10 @@
                                              #.instance-pointer-lowtag
                                              #.instance-slots-offset)))
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movq value (make-ea-for-raw-slot object tmp :index index))))
+    (inst movq value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-set/complex-single)
   (:translate %raw-instance-set/complex-single)
@@ -910,16 +832,11 @@
          (index :scs (any-reg))
          (value :scs (complex-single-reg) :target result))
   (:arg-types * positive-fixnum complex-single-float)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
     (move result value)
-    (inst movq (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits))) value)))
+    (inst movq (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-set-c/complex-single)
   (:translate %raw-instance-set/complex-single)
@@ -931,22 +848,19 @@
                                              #.instance-slots-offset))
               complex-single-float)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (complex-single-reg)))
   (:result-types complex-single-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
     (move result value)
-    (inst movq (make-ea-for-raw-slot object tmp :index index) value)))
+    (inst movq (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-init/complex-single)
   (:args (object :scs (descriptor-reg))
          (value :scs (complex-single-reg)))
   (:arg-types * complex-single-float)
-  (:info instance-length index)
+  (:info index)
   (:generator 4
-    (inst movq (make-ea-for-raw-slot object instance-length :index index) value)))
+    (inst movq (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-ref/complex-double)
   (:translate %raw-instance-ref/complex-double)
@@ -954,15 +868,10 @@
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
-    (inst movdqu value (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits)) :adjustment -8))))
+    (inst movdqu value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-ref-c/complex-double)
   (:translate %raw-instance-ref/complex-double)
@@ -972,13 +881,10 @@
                                              #.instance-pointer-lowtag
                                              #.instance-slots-offset)))
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (value :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst movdqu value (make-ea-for-raw-slot object tmp :index index :adjustment -8))))
+    (inst movdqu value (make-ea-for-raw-slot object index))))
 
 (define-vop (raw-instance-set/complex-double)
   (:translate %raw-instance-set/complex-double)
@@ -987,16 +893,11 @@
          (index :scs (any-reg))
          (value :scs (complex-double-reg) :target result))
   (:arg-types * positive-fixnum complex-double-float)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 5
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
-    (inst shl tmp n-fixnum-tag-bits)
-    (inst sub tmp index)
     (move result value)
-    (inst movdqu (make-ea-for-raw-slot object tmp :scale (ash 1 (- word-shift n-fixnum-tag-bits)) :adjustment -8) value)))
+    (inst movdqu (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-set-c/complex-double)
   (:translate %raw-instance-set/complex-double)
@@ -1008,19 +909,16 @@
                                              #.instance-slots-offset))
               complex-double-float)
   (:info index)
-  (:temporary (:sc unsigned-reg) tmp)
   (:results (result :scs (complex-double-reg)))
   (:result-types complex-double-float)
   (:generator 4
-    (loadw tmp object 0 instance-pointer-lowtag)
-    (inst shr tmp n-widetag-bits)
     (move result value)
-    (inst movdqu (make-ea-for-raw-slot object tmp :index index :adjustment -8) value)))
+    (inst movdqu (make-ea-for-raw-slot object index) value)))
 
 (define-vop (raw-instance-init/complex-double)
   (:args (object :scs (descriptor-reg))
          (value :scs (complex-double-reg)))
   (:arg-types * complex-double-float)
-  (:info instance-length index)
+  (:info index)
   (:generator 4
-    (inst movdqu (make-ea-for-raw-slot object instance-length :index index :adjustment -8) value)))
+    (inst movdqu (make-ea-for-raw-slot object index) value)))
