@@ -22,6 +22,16 @@
                      (nth n *register-arg-offsets*))
       (make-wired-tn *backend-t-primitive-type* control-stack-sc-number n)))
 
+(defun standard-arg-location-sc (n)
+  (declare (type unsigned-byte n))
+  (if (< n register-arg-count)
+      (make-sc-offset descriptor-reg-sc-number
+                      (nth n *register-arg-offsets*))
+      (make-sc-offset control-stack-sc-number n)))
+
+(defconstant arg-count-sc (make-sc-offset any-reg-sc-number ecx-offset))
+(defconstant closure-sc (make-sc-offset descriptor-reg-sc-number eax-offset))
+
 ;;; Make a passing location TN for a local call return PC.
 ;;;
 ;;; Always wire the return PC location to the stack in its standard
@@ -31,6 +41,9 @@
   (make-wired-tn (primitive-type-or-lose 'system-area-pointer)
                  sap-stack-sc-number return-pc-save-offset))
 
+(defconstant return-pc-passing-offset
+  (make-sc-offset sap-stack-sc-number return-pc-save-offset))
+
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in.
 ;;;
@@ -43,6 +56,9 @@
   (make-wired-tn *fixnum-primitive-type* control-stack-sc-number
                  ocfp-save-offset))
 
+(defconstant old-fp-passing-offset
+  (make-sc-offset control-stack-sc-number ocfp-save-offset))
+
 ;;; Make the TNs used to hold OLD-FP and RETURN-PC within the current
 ;;; function. We treat these specially so that the debugger can find
 ;;; them at a known location.
@@ -271,8 +287,7 @@
                               ancestor-frame-set/system-area-pointer))))
 
 (define-vop (xep-allocate-frame)
-  (:info start-lab copy-more-arg-follows)
-  (:vop-var vop)
+  (:info start-lab)
   (:generator 1
     (emit-alignment n-lowtag-bits)
     (emit-label start-lab)
@@ -283,18 +298,15 @@
 
     ;; The start of the actual code.
     ;; Save the return-pc.
-    (popw ebp-tn (frame-word-offset return-pc-save-offset))
+    (popw ebp-tn (frame-word-offset return-pc-save-offset))))
 
-    ;; If copy-more-arg follows it will allocate the correct stack
-    ;; size. The stack is not allocated first here as this may expose
-    ;; args on the stack if they take up more space than the frame!
-    (unless copy-more-arg-follows
-      ;; The args fit within the frame so just allocate the frame.
-      (inst lea esp-tn
+(define-vop (xep-setup-sp)
+  (:generator 1
+    (inst lea esp-tn
             (make-ea :dword :base ebp-tn
                      :disp (- (* n-word-bytes
                                  (- (max 3 (sb-allocated-size 'stack))
-                                    sp->fp-offset))))))))
+                                    sp->fp-offset)))))))
 
 ;;; This is emitted directly before either a known-call-local, call-local,
 ;;; or a multiple-call-local. All it does is allocate stack space for the
@@ -1128,14 +1140,6 @@
 
 ;;;; XEP hackery
 
-;;; We don't need to do anything special for regular functions.
-(define-vop (setup-environment)
-  (:info label)
-  (:ignore label)
-  (:generator 0
-    ;; Don't bother doing anything.
-    nil))
-
 ;;; Get the lexical environment from its passing location.
 (define-vop (setup-closure-environment)
   (:results (closure :scs (descriptor-reg)))
@@ -1331,6 +1335,18 @@
        (inst mov keyword (make-ea :dword :base object :index index
                                   :disp n-word-bytes))))))
 
+(define-vop (more-arg/c)
+  (:translate sb!c::%more-arg)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1)))
+  (:info index)
+  (:arg-types * (:constant (signed-byte 32)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst mov value (make-ea :dword :base object
+                                    :disp (- (* index n-word-bytes))))))
+
 (define-vop (more-arg)
     (:translate sb!c::%more-arg)
   (:policy :fast-safe)
@@ -1427,19 +1443,28 @@
 ;;; Signal wrong argument count error if NARGS isn't equal to COUNT.
 (define-vop (verify-arg-count)
   (:policy :fast-safe)
-  (:translate sb!c::%verify-arg-count)
   (:args (nargs :scs (any-reg)))
-  (:arg-types positive-fixnum (:constant t))
-  (:info count)
+  (:arg-types positive-fixnum (:constant t) (:constant t))
+  (:info min max)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
            (generate-error-code vop 'invalid-arg-count-error nargs)))
-      (if (zerop count)
-          (inst test nargs nargs)  ; smaller instruction
-        (inst cmp nargs (fixnumize count)))
-      (inst jmp :ne err-lab))))
+      (cond ((not min)
+             (if (zerop max)
+                 (inst test nargs nargs)
+                 (inst cmp nargs (fixnumize max)))
+             (inst jmp :ne err-lab))
+            (max
+             (when (plusp min)
+               (inst cmp nargs (fixnumize min))
+               (inst jmp :b err-lab))
+             (inst cmp nargs (fixnumize max))
+             (inst jmp :a err-lab))
+            ((plusp min)
+             (inst cmp nargs (fixnumize min))
+             (inst jmp :b err-lab))))))
 
 ;;; Various other error signallers.
 (macrolet ((def (name error translate &rest args)
@@ -1448,7 +1473,8 @@
                     `((:policy :fast-safe)
                       (:translate ,translate)))
                 (:args ,@(mapcar (lambda (arg)
-                                   `(,arg :scs (any-reg descriptor-reg)))
+                                   `(,arg :scs (any-reg descriptor-reg
+                                                control-stack constant)))
                                  args))
                 (:vop-var vop)
                 (:save-p :compute-only)
