@@ -509,11 +509,13 @@
 
 ;;; FIXME: find a better name.
 (defun !bootstrap-class-predicates (early-p)
-  (let ((*early-p* early-p))
-    (dolist (ecp *early-class-predicates*)
+  (let ((*early-p* early-p)
+        (source-loc (sb-c:source-location)))
+    (dolist (ecp *!early-class-predicates*)
       (let ((class-name (car ecp))
             (predicate-name (cadr ecp)))
-        (make-class-predicate (find-class class-name) predicate-name)))))
+        (!make-class-predicate (find-class class-name) predicate-name
+                               source-loc)))))
 
 (defun !bootstrap-built-in-classes ()
 
@@ -565,16 +567,12 @@
 
 (defun ensure-non-standard-class (name classoid &optional existing-class)
   (flet
-      ((ensure (metaclass &optional (slots nil slotsp))
+      ((ensure (metaclass slots)
          (let ((supers (mapcar #'classoid-name (classoid-direct-superclasses classoid))))
-           (if slotsp
-               (ensure-class-using-class existing-class name
-                                         :metaclass metaclass :name name
-                                         :direct-superclasses supers
-                                         :direct-slots slots)
-               (ensure-class-using-class existing-class name
-                                         :metaclass metaclass :name name
-                                         :direct-superclasses supers))))
+           (ensure-class-using-class existing-class name
+                                     :metaclass metaclass :name name
+                                     :direct-superclasses supers
+                                     :direct-slots slots)))
        (slot-initargs-from-structure-slotd (slotd)
          (let ((accessor (structure-slotd-accessor-symbol slotd)))
            `(:name ,(structure-slotd-name slotd)
@@ -616,9 +614,9 @@
 (pushnew 'ensure-deffoo-class sb-kernel::*defstruct-hooks*)
 (pushnew 'ensure-deffoo-class sb-kernel::*define-condition-hooks*)
 
-;;; FIXME: only needed during bootstrap
-(defun make-class-predicate (class name)
-  (let* ((gf (ensure-generic-function name :lambda-list '(object)))
+(defun !make-class-predicate (class name source-location)
+  (let* ((gf (ensure-generic-function name :lambda-list '(object)
+                                      :definition-source source-location))
          (mlist (if (eq **boot-state** 'complete)
                     (early-gf-methods gf)
                     (generic-function-methods gf))))
@@ -674,19 +672,13 @@
     (setq classoid (find-classoid classoid nil)))
   (etypecase classoid
     (null)
-    (built-in-classoid
-     (let ((translation (built-in-classoid-translation classoid)))
-       (cond
-         (translation
-          (aver (ctype-p translation))
-          (setf (info :type :translator class)
-                (lambda (spec) (declare (ignore spec)) translation)))
-         (t
-          (setf (info :type :translator class)
-                (lambda (spec) (declare (ignore spec)) classoid))))))
     (classoid
+     ;; There used to be an AVER preventing the placeholder :INITIALIZING from
+     ;; sneaking into globaldb. It can't any more due to type-safe (SETF INFO).
      (setf (info :type :translator class)
-           (lambda (spec) (declare (ignore spec)) classoid)))))
+           (or (and (typep classoid 'built-in-classoid)
+                    (built-in-classoid-translation classoid))
+               classoid)))))
 
 (!bootstrap-meta-braid)
 (!bootstrap-accessor-definitions t)
