@@ -98,9 +98,8 @@
                (slot-missing class object slot-name 'setf new-value)))))
 
 (defun compute-pv (slot-name-lists wrappers)
-  (unless (listp wrappers)
-    (setq wrappers (list wrappers)))
-  (let (elements)
+  (let ((wrappers (ensure-list wrappers))
+        elements)
     (dolist (slot-names slot-name-lists)
       (when slot-names
         (let* ((wrapper (pop wrappers))
@@ -108,18 +107,18 @@
                (class (wrapper-class* wrapper)))
           (dolist (slot-name (cdr slot-names))
             (let ((cell
-                   (or (find-slot-cell wrapper slot-name)
-                       (cons nil (slot-missing-info class slot-name)))))
+                    (or (find-slot-cell wrapper slot-name)
+                        (cons nil (slot-missing-info class slot-name)))))
               (push (when (and std-p (use-standard-slot-access-p class slot-name 'all))
                       (car cell))
-                  elements)
+                    elements)
               (push (or (cdr cell)
                         (bug "No SLOT-INFO for ~S in ~S" slot-name class))
-                  elements))))))
+                    elements))))))
     (let* ((n (length elements))
            (pv (make-array n)))
       (loop for i from (1- n) downto 0
-         do (setf (svref pv i) (pop elements)))
+            do (setf (svref pv i) (pop elements)))
       pv)))
 
 (defun pv-table-lookup (pv-table pv-wrappers)
@@ -156,6 +155,12 @@
       (finalize-inheritance class)
       t)))
 
+(declaim (ftype (sfunction (class) class) ensure-class-finalized))
+(defun ensure-class-finalized (class)
+  (unless (class-finalized-p class)
+    (finalize-inheritance class))
+  class)
+
 (defun can-optimize-access (form required-parameters env)
   (destructuring-bind (op var-form slot-name-form &optional new-value) form
     (let ((type (ecase op
@@ -182,7 +187,7 @@
                        (when (boundp 'sb-c:*lexenv*)
                          (sb-c:compiler-notify
                           "~@<Cannot optimize slot access, inheritance of ~S is not ~
-                           yet finaliable due to forward-referenced superclasses:~
+                           yet finalizable due to forward-referenced superclasses:~
                            ~%  ~S~:@>"
                           class form))
                        (setf class nil))))
