@@ -38,7 +38,7 @@ lispobj *current_binding_stack_pointer;
 
 /* ALLOCATION_POINTER is x86 or RT.  Anyone want to do an RT port?   */
 
-# if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
+# if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64) && !defined(LISP_FEATURE_ARM)
 /* The Object Formerly Known As current_dynamic_space_free_pointer */
 lispobj *dynamic_space_free_pointer;
 #endif
