@@ -324,38 +324,24 @@ static void print_fixnum(lispobj obj)
 static void brief_otherimm(lispobj obj)
 {
     int type, c;
-    char buffer[10];
+    char * charname = 0;
 
     type = widetag_of(obj);
     switch (type) {
         case CHARACTER_WIDETAG:
-            c = (obj>>8)&0xff;
+            c = obj>>8; // no mask. show whatever's there
+            printf("#\\");
             switch (c) {
-                case '\0':
-                    printf("#\\Null");
-                    break;
-                case '\n':
-                    printf("#\\Newline");
-                    break;
-                case '\b':
-                    printf("#\\Backspace");
-                    break;
-                case '\177':
-                    printf("#\\Delete");
-                    break;
+                case '\0': charname = "Nul"; break;
+                case '\n': charname = "Newline"; break;
+                case '\b': charname = "Backspace"; break;
+                case '\177': charname = "Delete"; break;
                 default:
-                    strcpy(buffer, "#\\");
-                    if (c >= 128) {
-                        strcat(buffer, "m-");
-                        c -= 128;
-                    }
-                    if (c < 32) {
-                        strcat(buffer, "c-");
-                        c += '@';
-                    }
-                    printf("%s%c", buffer, c);
-                    break;
+                  if (c < 32) printf("^%c", c+64);
+                  else printf(c < 128 ? "%c" : "U+%X", c);
             }
+            if (charname)
+                fputs(charname, stdout);
             break;
 
         case UNBOUND_MARKER_WIDETAG:
@@ -383,7 +369,7 @@ static void print_otherimm(lispobj obj)
             break;
 
         default:
-            printf(": data=%ld", (long) (obj>>8)&0xffffff);
+            printf(": data=%"OBJ_FMTX, (obj>>8));
             break;
     }
 }
@@ -437,17 +423,66 @@ static void print_list(lispobj obj)
     }
 }
 
+// takes native pointer as input
+char * simple_base_stringize(struct vector * string)
+{
+  if (widetag_of(string->header) == SIMPLE_BASE_STRING_WIDETAG)
+      return (char*)string->data;
+  int length = string->length;
+  char * newstring = malloc(length+1);
+  uint32_t * data = (uint32_t*)string->data;
+  int i;
+  for(i=0;i<length;++i)
+      newstring[i] = data[i] < 128 ? data[i] : '?';
+  newstring[length] = 0;
+  return newstring;
+}
+
 static void brief_struct(lispobj obj)
 {
     struct instance *instance = (struct instance *)native_pointer(obj);
     if (!is_valid_lisp_addr((os_vm_address_t)instance)) {
         printf("(invalid address)");
     } else {
-        printf("#<ptr to 0x%08lx instance>",
-               (unsigned long) instance->slots[0]);
+        extern struct vector * instance_classoid_name(lispobj*);
+        struct vector * classoid_name;
+        classoid_name = instance_classoid_name((lispobj*)instance);
+        if ( classoid_name ) {
+          char * namestring = simple_base_stringize(classoid_name);
+          printf("#<ptr to %p %s instance>",
+                 (void*)instance_layout((lispobj*)instance), namestring);
+          if ( namestring != (char*)classoid_name->data )
+              free(namestring);
+        } else {
+          printf("#<ptr to %p instance>",
+                 (void*)instance_layout((lispobj*)instance));
+        }
     }
 }
 
+#include "genesis/layout.h"
+static boolean untagged_slot_p(struct layout * layout,
+                               int slot_index)
+{
+#ifdef LISP_FEATURE_INTERLEAVED_RAW_SLOTS
+  extern boolean positive_bignum_logbitp(int,struct bignum*);
+  lispobj bitmap = layout->untagged_bitmap;
+  return fixnump(bitmap)
+         ? (fixnum_value(bitmap) >> slot_index) & 1
+         : positive_bignum_logbitp(slot_index,
+                                   (struct bignum*)native_pointer(bitmap));
+#else
+  // STANDARD-OBJECT has layout-length = number of defined slots, but
+  // the primitive object always occupies 4 physical words. The guard on
+  // n_untagged_slots ensures that for classes with no slots, we don't
+  // wrongly show all words of the primitive object as untagged,
+  // because the second half of the expression reduces to slot_index>=1.
+  return layout->n_untagged_slots > 0
+         && slot_index >= (fixnum_value(layout->length)|1)
+                          - fixnum_value(layout->n_untagged_slots);
+#endif
+}
+
 static void print_struct(lispobj obj)
 {
     struct instance *instance = (struct instance *)native_pointer(obj);
@@ -456,21 +491,58 @@ static void print_struct(lispobj obj)
     if (!is_valid_lisp_addr((os_vm_address_t)instance)) {
         printf("(invalid address)");
     } else {
-        print_obj("type: ", ((struct instance *)native_pointer(obj))->slots[0]);
-        for (i = 1; i < HeaderValue(instance->header); i++) {
+        lispobj layout_obj =  instance_layout(native_pointer(obj));
+        print_obj("type: ", layout_obj);
+        struct layout * layout = (struct layout*)native_pointer(layout_obj);
+        for (i=INSTANCE_DATA_START; i<instance_length(instance->header); i++) {
             sprintf(buffer, "slot %d: ", i);
-            print_obj(buffer, instance->slots[i]);
+            if (layout==NULL || untagged_slot_p(layout, i)) {
+                newline(NULL);
+                printf("\n\t    %s0x%"OBJ_FMTX" [raw]", buffer, instance->slots[i]);
+            } else
+                print_obj(buffer, instance->slots[i]);
         }
     }
 }
 
+void show_lstring(struct vector * string, int quotes, FILE *s)
+{
+  int ucs4_p = 0;
+  int i, len = fixnum_value(string->length);
+
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
+  if (widetag_of(string->header) == SIMPLE_CHARACTER_STRING_WIDETAG) {
+      ucs4_p = 1;
+      if (quotes)
+          putc('u', s); /* an arbitrary notational convention */
+  }
+#endif
+  if (quotes) putc('"', s);
+  for (i=0 ; i<len ; i++) {
+      // hopefully the compiler will optimize out the ucs4_p test
+      // when the runtime is built without Unicode support
+      int ch;
+      if (ucs4_p)
+          ch = i[(uint32_t*)string->data];
+      else
+          ch = i[(char*)string->data];
+      if (ch >= 32 && ch < 127) {
+          if (quotes && (ch == '"' || ch == '\\'))
+              putc('\\', s);
+          putc(ch, s);
+      } else {
+          fprintf(s, ch > 0xffff ? "\\U%08X" :
+                     ch > 0xff ? "\\u%04X" : "\\x%02X", ch);
+      }
+  }
+  if (quotes) putc('"', s);
+}
+
 static void brief_otherptr(lispobj obj)
 {
     lispobj *ptr, header;
     int type;
     struct symbol *symbol;
-    struct vector *vector;
-    char *charptr;
 
     ptr = (lispobj *) native_pointer(obj);
 
@@ -484,48 +556,18 @@ static void brief_otherptr(lispobj obj)
     switch (type) {
         case SYMBOL_HEADER_WIDETAG:
             symbol = (struct symbol *)ptr;
-            vector = (struct vector *)native_pointer(symbol->name);
-            for (charptr = (char *)vector->data; *charptr != '\0'; charptr++) {
-                if (*charptr == '"')
-                    putchar('\\');
-                putchar(*charptr);
-            }
+            if (symbol->package == NIL)
+                printf("#:");
+            show_lstring((struct vector *)native_pointer(symbol->name),
+                         0, stdout);
             break;
 
         case SIMPLE_BASE_STRING_WIDETAG:
-            vector = (struct vector *)ptr;
-            putchar('"');
-            for (charptr = (char *)vector->data; *charptr != '\0'; charptr++) {
-                if (*charptr == '"')
-                    putchar('\\');
-                putchar(*charptr);
-            }
-            putchar('"');
-            break;
-
-#ifdef LISP_FEATURE_SB_UNICODE
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
         case SIMPLE_CHARACTER_STRING_WIDETAG:
-            vector = (struct vector *)ptr;
-            fputs("u\"", stdout);
-            {
-                int i, ch, len = fixnum_value(vector->length);
-                uint32_t *chars = (uint32_t*)vector->data;
-                for (i=0 ; i<len ; i++) {
-                    ch = chars[i];
-                    if (ch >= 32 && ch < 127) {
-                        if (ch == '"' || ch == '\\')
-                            putchar('\\');
-                        putchar(ch);
-                    } else {
-                      // ambiguous, e.g. #\xaaa is either #\GUJARATI_LETTER_PA
-                      // or #\FEMININE_ORDINAL_INDICATOR + #\a. oh well.
-                      printf("\\x%x", ch);
-                    }
-                }
-            }
-            putchar('"');
-            break;
 #endif
+            show_lstring((struct vector*)ptr, 1, stdout);
+            break;
 
         default:
             printf("#<ptr to ");
@@ -558,9 +600,9 @@ static char *ratio_slots[] = {"numer: ", "denom: ", NULL};
 static char *complex_slots[] = {"real: ", "imag: ", NULL};
 static char *code_slots[] = {"words: ", "entry: ", "debug: ", NULL};
 static char *fn_slots[] = {
-    "self: ", "next: ", "name: ", "arglist: ", "type: ", NULL};
+    "self: ", "next: ", "name: ", "arglist: ", "type: ", "info: ", NULL};
 static char *closure_slots[] = {"fn: ", NULL};
-static char *funcallable_instance_slots[] = {"fn: ", "lexenv: ", "layout: ", NULL};
+static char *funcallable_instance_slots[] = {"raw_fn: ", "fn: ", "layout: ", NULL};
 static char *weak_pointer_slots[] = {"value: ", NULL};
 static char *fdefn_slots[] = {"name: ", "function: ", "raw_addr: ", NULL};
 static char *value_cell_slots[] = {"value: ", NULL};
@@ -580,7 +622,7 @@ static void print_otherptr(lispobj obj)
         u32 length;
 #endif
         int count, type, index;
-        char *cptr, buffer[16];
+        char buffer[16];
 
         ptr = (lispobj*) native_pointer(obj);
         if (ptr == NULL) {
@@ -600,13 +642,21 @@ static void print_otherptr(lispobj obj)
             return;
         }
 
+        if (unprintable_array_types[type/8] & (1<<(type % 8)))
+            return;
         switch (type) {
             case BIGNUM_WIDETAG:
                 ptr += count;
                 NEWLINE_OR_RETURN;
                 printf("0x");
                 while (count-- > 0)
-                    printf("%08lx", (unsigned long) *--ptr);
+                    printf(
+#if N_WORD_BITS == 32
+                           "%08lx%s",
+#else
+                           "%016lx%s",
+#endif
+                           (unsigned long) *--ptr, (count?"_":""));
                 break;
 
             case RATIO_WIDETAG:
@@ -678,14 +728,10 @@ static void print_otherptr(lispobj obj)
 
             case SIMPLE_BASE_STRING_WIDETAG:
 #ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-        case SIMPLE_CHARACTER_STRING_WIDETAG: /* FIXME */
+            case SIMPLE_CHARACTER_STRING_WIDETAG:
 #endif
                 NEWLINE_OR_RETURN;
-                cptr = (char *)(ptr+1);
-                putchar('"');
-                while (length-- > 0)
-                    putchar(*cptr++);
-                putchar('"');
+                show_lstring((struct vector*)native_pointer(obj), 1, stdout);
                 break;
 
             case SIMPLE_VECTOR_WIDETAG:
@@ -699,6 +745,7 @@ static void print_otherptr(lispobj obj)
                 }
                 break;
 
+            // FIXME: This case looks unreachable. print_struct() does it
             case INSTANCE_HEADER_WIDETAG:
                 NEWLINE_OR_RETURN;
                 printf("length = %ld", (long) count);
@@ -709,75 +756,19 @@ static void print_otherptr(lispobj obj)
                 }
                 break;
 
-            case SIMPLE_ARRAY_WIDETAG:
-            case SIMPLE_BIT_VECTOR_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_4_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_7_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_8_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_15_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_16_WIDETAG:
-
-            case SIMPLE_ARRAY_UNSIGNED_FIXNUM_WIDETAG:
-
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_31_WIDETAG:
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG:
-#ifdef SIMPLE_ARRAY_UNSIGNED_BYTE_63_WIDETAG
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_63_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_UNSIGNED_BYTE_64_WIDETAG
-            case SIMPLE_ARRAY_UNSIGNED_BYTE_64_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_SIGNED_BYTE_8_WIDETAG
-            case SIMPLE_ARRAY_SIGNED_BYTE_8_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_SIGNED_BYTE_16_WIDETAG
-            case SIMPLE_ARRAY_SIGNED_BYTE_16_WIDETAG:
-#endif
-
-            case SIMPLE_ARRAY_FIXNUM_WIDETAG:
-
-#ifdef SIMPLE_ARRAY_SIGNED_BYTE_32_WIDETAG
-            case SIMPLE_ARRAY_SIGNED_BYTE_32_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_SIGNED_BYTE_64_WIDETAG
-            case SIMPLE_ARRAY_SIGNED_BYTE_64_WIDETAG:
-#endif
-            case SIMPLE_ARRAY_SINGLE_FLOAT_WIDETAG:
-            case SIMPLE_ARRAY_DOUBLE_FLOAT_WIDETAG:
-#ifdef SIMPLE_ARRAY_LONG_FLOAT_WIDETAG
-            case SIMPLE_ARRAY_LONG_FLOAT_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_COMPLEX_SINGLE_FLOAT_WIDETAG
-            case SIMPLE_ARRAY_COMPLEX_SINGLE_FLOAT_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_COMPLEX_DOUBLE_FLOAT_WIDETAG
-            case SIMPLE_ARRAY_COMPLEX_DOUBLE_FLOAT_WIDETAG:
-#endif
-#ifdef SIMPLE_ARRAY_COMPLEX_LONG_FLOAT_WIDETAG
-            case SIMPLE_ARRAY_COMPLEX_LONG_FLOAT_WIDETAG:
-#endif
-            case COMPLEX_BASE_STRING_WIDETAG:
-#ifdef COMPLEX_CHARACTER_STRING_WIDETAG
-        case COMPLEX_CHARACTER_STRING_WIDETAG:
-#endif
-            case COMPLEX_VECTOR_NIL_WIDETAG:
-            case COMPLEX_BIT_VECTOR_WIDETAG:
-            case COMPLEX_VECTOR_WIDETAG:
-            case COMPLEX_ARRAY_WIDETAG:
-                break;
-
             case CODE_HEADER_WIDETAG:
                 print_slots(code_slots, count-1, ptr);
                 break;
 
             case SIMPLE_FUN_HEADER_WIDETAG:
-                print_slots(fn_slots, 5, ptr);
+                print_slots(fn_slots, 6, ptr);
                 break;
 
+#if !defined(LISP_FEATURE_X86) && !defined(LISP_FEATURE_X86_64)
             case RETURN_PC_HEADER_WIDETAG:
                 print_obj("code: ", obj - (count * 4));
                 break;
+#endif
 
             case CLOSURE_HEADER_WIDETAG:
                 print_slots(closure_slots, count, ptr);
