@@ -90,12 +90,22 @@
 
 ;;; ARRAY-IN-BOUNDS-P should work when given non-INDEXes as its
 ;;; subscripts (and return NIL, of course)
-(let ((a (make-array 10 :fill-pointer 5)))
-  (assert (not (array-in-bounds-p a -1)))
-  (assert (array-in-bounds-p a 3))
-  (assert (array-in-bounds-p a 7))
-  (assert (not (array-in-bounds-p a 11)))
-  (assert (not (array-in-bounds-p a (1+ most-positive-fixnum)))))
+(with-test (:name array-in-bounds-p)
+  (macrolet
+      ((test-case (array subscript expected)
+         `(progn
+            (assert (,(if expected 'progn 'not)
+                      (array-in-bounds-p ,array ,subscript)))
+            (assert (,(if expected 'progn 'not)
+                      (funcall (compile nil '(lambda (array subscript)
+                                              (array-in-bounds-p array subscript)))
+                               ,array ,subscript))))))
+   (let ((a (make-array 10 :fill-pointer 5)))
+     (test-case a -1                        nil)
+     (test-case a  3                        t)
+     (test-case a  7                        t)
+     (test-case a 11                        nil)
+     (test-case a (1+ most-positive-fixnum) nil))))
 
 ;;; arrays of bits should work:
 (let ((a (make-array '(10 10) :element-type 'bit :adjustable t)))
@@ -312,8 +322,28 @@
        (string= (simple-condition-format-control c)
                 "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS")))))
 
+(with-test (:name :make-array-sanity-check-dims-first)
+  ;; A full call to %MAKE-ARRAY will signal a TYPE-ERROR on these inputs
+  ;; instead of trying to consume a massive amount of memory.
+  ;; Additionally, the relevent IR1 transform should give up.
+  (locally
+    (declare (notinline make-array))
+    (assert-error (make-array `(-1 -1 ,(- (ash array-dimension-limit -2) 4)))
+                  type-error))
+  (locally
+    (declare (inline make-array))
+    (assert-error (make-array `(-1 -1 ,(- (ash array-dimension-limit -2) 4)))
+                  type-error)))
+
 (with-test (:name :make-array-size-overflow)
   ;; 1-bit fixnum tags make array limits overflow the word length
   ;; when converted to bytes
   (when (= sb-vm:n-fixnum-tag-bits 1)
     (assert-error (make-array (1- array-total-size-limit)) error)))
+
+(with-test (:name :adjust-non-adjustable-array)
+  (let* ((a (make-array '(2 3) :initial-contents '((0 1 2) (3 4 5))))
+         (b (adjust-array a '(2 2))))
+    (setf (aref a 0 0) 11)
+    (assert (zerop (aref b 0 0)))
+    (assert (not (eq a b)))))
