@@ -151,6 +151,32 @@
     (sb-ext:atomic-decf (box-word box) 2)
     (assert (= (- (ash 1 sb-vm:n-word-bits) 2) (box-word box)))))
 
+(with-test (:name :atomic-incf-full-call-lp1381867
+            :skipped-on '(not (or :x86 :x86-64 :ppc)))
+  ;; contortions to avoid reader errors
+  (let* ((%riai/w (intern "%RAW-INSTANCE-ATOMIC-INCF/WORD" "SB-KERNEL"))
+         (form
+          `(locally
+            ;; Rebind %RAW-INSTANCE-ATOMIC-INCF/WORD as a local
+            ;; function.  Declaring it locally notinline fails because
+            ;; it is marked with the ALWAYS-TRANSLATABLE attribute, so
+            ;; it's a bug to call it even though we've asked to call
+            ;; it. (Maybe that's a bug?)  And I don't want to call it
+            ;; explictly - I want the macro to do it so that I don't
+            ;; have to inject any of the sign making noise and such.
+            (declare (disable-package-locks ,%riai/w))
+            (let ((b (make-box :word 0))
+                  (delta (- (ash 1 (1- sb-vm:n-word-bits))))
+                  (f (fdefinition ',%riai/w)))
+              (flet ((,%riai/w (a b c) (funcall f a b c)))
+                (assert (= (atomic-incf (box-word b) delta) 0))
+                (assert (= (atomic-incf (box-word b) delta)
+                           (ash 1 (1- sb-vm:n-word-bits))))
+                (assert (= (box-word b) 0))
+                (atomic-decf (box-word b))
+                (assert (= (box-word b) sb-ext:most-positive-word)))))))
+    (eval form)))
+
 #+sb-thread
 (with-test (:name (:atomic-incf/decf :threads))
   (let* ((box (make-box))
@@ -453,11 +479,13 @@
 (with-test (:name :symbol-value-cas-expansion)
   (multiple-value-bind (vars vals old new cas-form read-form)
       (get-cas-expansion `(symbol-value t))
+    (declare (ignore old new cas-form))
     (assert (not vars))
     (assert (not vals))
     (assert (eq t (eval read-form))))
   (multiple-value-bind (vars vals old new cas-form read-form)
       (get-cas-expansion `(symbol-value *))
+    (declare (ignore old new cas-form))
     (let ((* :foo))
       (assert (eq :foo
                   (eval `(let (,@(mapcar 'list vars vals))
