@@ -560,6 +560,14 @@
                    (:no-error (res)
                      (1- res))))))
 
+(defun list-delete-some-stuff ()
+  ;; opaque-identity hides the fact that we are calling a destructive function
+  ;; on a constant, which is technically illegal. But no deletion occurs,
+  ;; so it's innocuous.
+  (delete 'a (opaque-identity '(x y)))
+  (delete 'a (opaque-identity '(x y)) :from-end t)
+  (delete-duplicates (opaque-identity '(x y))))
+
 (defvar *a-cons* (cons nil nil))
 
 (with-test (:name (:no-consing :dx-closures) :skipped-on '(not :stack-allocatable-closures))
@@ -578,6 +586,7 @@
   (assert-no-consing (nested-evil-dx-used *a-cons*))
   (assert-no-consing (mapcar-negate nil))
   (assert-no-consing (mapcan-reverse nil))
+  (assert-no-consing (list-delete-some-stuff))
   (assert-no-consing (multiple-dx-uses)))
 
 (with-test (:name (:no-consing :dx-value-cell))
@@ -600,7 +609,6 @@
   (assert-no-consing (vector-on-stack :x :y)))
 
 (with-test (:name (:no-consing :specialized-dx-vectors)
-            :fails-on :x86
             :skipped-on `(not (and :stack-allocatable-vectors
                                    :c-stack-is-control-stack)))
   (assert-no-consing (make-array-on-stack-1))
@@ -993,7 +1001,28 @@
         (return (bar))))))
 (with-test (:name :bug-681092)
   (assert (= 10 (bug-681092))))
-
+
+;;;; Including a loop in the flow graph between a DX-allocation and
+;;;; the start of its environment would, for a while, cause executing
+;;;; any of the code in the loop to discard the value.  Found via
+;;;; attempting to DX-allocate an array, which triggered the FILL
+;;;; transform, which inserted such a loop.
+(defun bug-1472785 (x)
+  (let ((y (let ((z (cons nil nil)))
+             (let ((i 0))
+               (tagbody
+                b1
+                  (when (= x i) (go b2))
+                  (incf i)
+                  (go b1)
+                b2))
+             z))
+        (w (cons t t)))
+    (declare (dynamic-extent y w))
+    (eq y w)))
+(with-test (:name :bug-1472785)
+  (assert (null (bug-1472785 1))))
+
 ;;;; &REST lists should stop DX propagation -- not required by ANSI,
 ;;;; but required by sanity.
 
