@@ -13,10 +13,10 @@
 
 ; WHITE-BOX TESTS
 
-(in-package "SB-THREAD")
-(shadowing-import 'assertoid:assert-error)
-(use-package :test-util)
-(use-package "ASSERTOID")
+(cl:in-package #:sb-thread)
+(cl:shadowing-import 'assertoid:assert-error)
+(cl:use-package '#:test-util)
+(cl:use-package '#:assertoid)
 
 (setf sb-unix::*on-dangerous-wait* :error)
 
@@ -151,28 +151,41 @@
                                                              (svref x 1)))
 (format t "~&compare-and-swap tests done~%")
 
-(with-test (:name (:threads :more-trivia)))
-(let ((old-threads (list-all-threads))
-      (thread (make-thread (lambda ()
-                             (assert (find *current-thread* *all-threads*))
-                             (sleep 2))))
-      (new-threads (list-all-threads)))
-  (assert (thread-alive-p thread))
-  (assert (eq thread (first new-threads)))
-  (assert (= (1+ (length old-threads)) (length new-threads)))
-  (sleep 3)
-  (assert (not (thread-alive-p thread))))
+(with-test (:name (:threads :more-trivia))
+  (let ((old-threads (list-all-threads))
+        (thread (make-thread (lambda ()
+                               (assert (find *current-thread* *all-threads*))
+                               (sleep 2))))
+        (new-threads (list-all-threads)))
+    (assert (thread-alive-p thread))
+    (assert (eq thread (first new-threads)))
+    (assert (= (1+ (length old-threads)) (length new-threads)))
+    (sleep 3)
+    (assert (not (thread-alive-p thread)))))
 
-(with-test (:name (:join-thread :nlx :default))
-  (let ((sym (gensym)))
-    (assert (eq sym (join-thread (make-thread (lambda () (sb-thread:abort-thread)))
-                                 :default sym)))))
+(with-test (:name (join-thread :abort :default))
+  (let* ((sym (gensym))
+         (thread (make-thread (lambda () (abort-thread)))))
+    (assert (equal (multiple-value-list
+                    (join-thread thread :default sym))
+                   (list sym :abort)))))
 
-(with-test (:name (:join-thread :nlx :error))
-  (assert-error (join-thread (make-thread (lambda () (sb-thread:abort-thread))))
+(with-test (:name (join-thread :abort :error))
+  (assert-error (join-thread (make-thread (lambda () (abort-thread))))
                 join-thread-error))
 
-(with-test (:name (:join-thread :multiple-values))
+(with-test (:name (join-thread :timeout :default))
+  (let* ((sym (gensym))
+         (thread (make-kill-thread (lambda () (sleep most-positive-fixnum)))))
+    (assert (equal (multiple-value-list
+                    (join-thread thread :timeout .001 :default sym))
+                   (list sym :timeout)))))
+
+(with-test (:name (join-thread :timeout :error))
+  (let ((thread (make-kill-thread (lambda () (sleep most-positive-fixnum)))))
+    (assert-error (join-thread thread :timeout .001) join-thread-error)))
+
+(with-test (:name (join-thread :multiple-values))
   (assert (equal '(1 2 3)
                  (multiple-value-list
                   (join-thread (make-thread (lambda () (values 1 2 3))))))))
@@ -193,6 +206,7 @@
       (funcall function)
       (prog1 (- (get-internal-run-time) start-time)
         (sb-thread:condition-broadcast queue)))))
+
 (defun fact (n)
   "A function that does work with the CPU."
   (if (zerop n) 1 (* n (fact (1- n)))))
@@ -428,7 +442,7 @@
 (with-test (:name (:semaphore :initial-count))
   (let ((sem (make-semaphore :count 1)))
     (sb-ext:with-timeout 0.1
-      (wait-on-semaphore sem))))
+      (assert (= 0 (wait-on-semaphore sem))))))
 
 (with-test (:name (:semaphore :wait-then-signal))
   (let ((sem (make-semaphore))
@@ -437,7 +451,7 @@
                         (sleep 0.1)
                         (setq signalled-p t)
                         (signal-semaphore sem)))
-    (wait-on-semaphore sem)
+    (assert (= 0 (wait-on-semaphore sem)))
     (assert signalled-p)))
 
 (with-test (:name (:semaphore :signal-then-wait))
@@ -447,7 +461,7 @@
                         (signal-semaphore sem)
                         (setq signalled-p t)))
     (loop until signalled-p)
-    (wait-on-semaphore sem)
+    (assert (= 0 (wait-on-semaphore sem)))
     (assert signalled-p)))
 
 (defun test-semaphore-multiple-signals (wait-on-semaphore)
@@ -477,7 +491,7 @@
 
 (with-test (:name (:try-semaphore :trivial-success))
   (let ((sem (make-semaphore :count 1)))
-    (assert (try-semaphore sem))
+    (assert (= 0 (try-semaphore sem)))
     (assert (zerop (semaphore-count sem)))))
 
 (with-test (:name (:try-semaphore :trivial-fail :n>1))
@@ -485,8 +499,8 @@
 
 (with-test (:name (:try-semaphore :trivial-success :n>1))
   (let ((sem (make-semaphore :count 10)))
-    (assert (try-semaphore sem 5))
-    (assert (try-semaphore sem 5))
+    (assert (= 5 (try-semaphore sem 5)))
+    (assert (= 0 (try-semaphore sem 5)))
     (assert (zerop (semaphore-count sem)))))
 
 (with-test (:name (:try-semaphore :emulate-wait-on-semaphore))
@@ -530,8 +544,6 @@
       (assert (zerop (count-live-threads waiters)))
       (assert (zerop (count-live-threads more-waiters))))))
 
-
-
 (format t "~&semaphore tests done~%")
 
 (defun test-interrupt (function-to-interrupt &optional quit-p)
